Package: EnMCB
Type: Package
Title: Predicting Disease Progression Based on Methylation Correlated
        Blocks using Ensemble Models
Version: 1.22.0
Date: 2023-3-13
Author: Xin Yu 
Maintainer: Xin Yu <whirlsyu@gmail.com>
Depends: R (>= 4.0)
Encoding: UTF-8
Imports: survivalROC, glmnet, rms, mboost, Matrix, igraph, methods,
        survivalsvm, ggplot2, boot, e1071, survival, BiocFileCache
ImportFrom:
VignetteBuilder: knitr
Suggests: SummarizedExperiment, testthat, Biobase, survminer,
        affycoretools, knitr, plotROC, limma, rmarkdown
Description: Creation of the correlated blocks using DNA methylation profiles. 
    Machine learning models can be constructed to predict differentially methylated blocks and disease progression.
License: GPL-2
BugReports: https://github.com/whirlsyu/EnMCB/issues
biocViews: Normalization, DNAMethylation, MethylationArray,
        SupportVectorMachine
LazyData: FALSE
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/EnMCB
git_branch: RELEASE_3_22
git_last_commit: d87a235
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:47:14 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,ridgemat)
S3method(predict,mcb.coxph.penal)
export(CompareMCB)
export(DiffMCB)
export(IdentifyMCB)
export(IdentifyMCB_parallel)
export(as.ridgemat)
export(create_demo)
export(draw_survival_curve)
export(ensemble_model)
export(ensemble_prediction)
export(fast_roc_calculation)
export(metricMCB)
export(metricMCB.cv)
export(multi_coxph)
export(pre_process_methylation)
export(univ_coxph)
import(BiocFileCache)
import(ggplot2)
import(glmnet)
import(stats)
import(survival)
import(survivalROC)
import(survivalsvm)
import(utils)
importFrom(mboost,CoxPH)
importFrom(mboost,boost_control)
importFrom(mboost,glmboost)
importFrom(mboost,predict.glmboost)
importFrom(methods,as)

# EnMCB
Package: EnMCB

Type: Package

Title: Predicting Disease Progression Based on Methylation Correlated Blocks using Ensemble Models
        
Version: 1.11.1

Author: Xin Yu

Maintainer: Xin Yu <whirlsyu@gmail.com> <yuxin@qlnu.edu.cn>

Description: This package is designed to help you to create the methylation correlated blocks using methylation profiles. A stacked ensemble of machine learning models, which combined the Cox regression, support vector regression, Coxboost and elastic-net regression model, can be constructed using this package. You also can choose one of them to build DNA methylation signatures associated with disease progression (survival).

License: GPL-3

Citation:
Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415


Followings are brief instructions for using this package:

You can install and our package via BiocManager as following or downloading source from github.

<pre>
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("EnMCB")
</pre>


First, you need a methylation data set, currently only most common platform 'Illumina Infinium Human Methylation 450K' is supported. For other platform, for instance, BS sequencing data, users need to 
introduce the annonation file for other own.

You can use your own datasets:

<pre>
eset_met<-some_methylation_datamatrix
</pre>

or use our demo data.

<pre>
data('demo_data',package = "EnMCB")
methylation_dataset<-demo_data$realdata
</pre>


Then, you can automatically run following:

<pre>
library(SummarizedExperiment)

res<-IdentifyMCB(methylation_dataset)

</pre>

You can extract the MCB information,

<pre>

MCB<-res$MCBinformation

</pre>

and select some of MCBs for further modeling.

<pre>

MCB<-MCB[MCB[,"CpGs_num"]>5,]

</pre>

In order to get differentially methylated blocks, one may run following:

<pre>
#simulation for the group data
groups = c(rep("control",200),rep("dis",255))

diffMCB_results <- DiffMCB(methylation_dataset, groups, MCB)

</pre>

In order to build survival models, one may run following:

<pre>
data(demo_survival_data)
survival_data <- demo_survival_data
# sample the dataset into training set and testing set
trainingset<-colnames(methylation_dataset) %in% sample(colnames(methylation_dataset),0.6*length(colnames(methylation_dataset)))

testingset<-!trainingset

#build the models

models<-metricMCB(MCB,
                    training_set=methylation_dataset[,training_set],
                    Surv=survival_data)

#select the best
onemodel<-models$best_cox_model

</pre>                    

Then, you can predict the risk by the model you build:

<pre>
predict(onemodel,methylation_dataset[,testing_set])
</pre>

In order to build ensemble model, one may run following:

<pre>
# You can choose one of MCBs:
select_single_one=1

em<-ensemble_model(MCB[select_single_one,],
                    training_set=methylation_dataset[,training_set],
                    Surv=survival_data)
                    
</pre>
Note that this function only can be used for single MCB only, otherwise the precessing time could be very long.

Then, you can predict the risk by the model you build:

<pre>
ensemble_prediction(ensemble_model = em,
                    predition_data = methylation_dataset[,testing_set])
</pre>

For detailed information, you can find at our references.

#' @title Trainging stacking ensemble model for Methylation Correlation Block
#'
#' @description Method for training a stacking ensemble model for Methylation Correlation Block.
#' @export
#' @param single_res Methylation Correlation Block information returned by the IndentifyMCB function.
#' @param training_set methylation matrix used for training the model in the analysis.
#' @param Surv_training Survival function contain the survival information for training.
#' @param testing_set methylation matrix used for testing the model in the analysis.
#' @param Surv_testing Survival function contain the survival information for testing.
#' @param ensemble_type Secondary model use for ensemble, one of "Cox", "C-index" and "feature weighted linear regression".
#' "feature weighted linear regression" only uses two meta-features namely kurtosis and S.D.
#' @author Xin Yu
#' @keywords methylation ensemble stacking
#' @usage ensemble_model(single_res,training_set,Surv_training,testing_set,
#' Surv_testing,ensemble_type)
#' @return Object of class \code{list} with elements (XXX repesents the model you choose):
#'  \tabular{ll}{
#'    \code{cox} \tab Model object for the cox model at first level. \cr
#'    \code{svm} \tab Model object for the svm model at first level. \cr
#'    \code{enet} \tab Model object for the enet model at first level. \cr
#'    \code{mboost} \tab Model object for the mboost model at first level. \cr
#'    \code{stacking} \tab Model object for the stacking model. \cr
#'  }
#' @references
#'  Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
#' @examples
#' #import datasets
#' library(survival)
#' data(demo_survival_data)
#' datamatrix<-create_demo()
#' data(demo_MCBinformation)
#' #select MCB with at least 3 CpGs.
#' demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]
#' trainingset<-colnames(datamatrix) %in% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
#' select_single_one=1
#' em<-ensemble_model(t(demo_MCBinformation[select_single_one,]),
#'     training_set=datamatrix[,trainingset],
#'     Surv_training=demo_survival_data[trainingset])
#'
#'
ensemble_model <- function(single_res, 
                           training_set, 
                           Surv_training, 
                           testing_set=NULL, 
                           Surv_testing=NULL,
                           ensemble_type = "Standard Cox") {
  if (dim(single_res)[1]>dim(single_res)[2]) {
    single_res<-t(as.matrix(single_res))
  }
  if (nrow(single_res)!=1){
    stop(errorCondition("ERROR: Results information (single_res) should only contain one single Methylation Correlation Block."))
  }
  rz=!(is.na(Surv_training)|Surv_training[,1]==0)
  Surv_training<-Surv_training[rz]
  all_related_CpGs<-strsplit(paste(single_res[,"CpGs"],collapse = " ")," ")[[1]]
  related_training<-training_set[rownames(training_set) %in% all_related_CpGs,rz]
  if (!is.null(testing_set)) {
    related_testing<-testing_set[rownames(testing_set) %in% all_related_CpGs,]
  }else{
    related_testing<-NULL
  }
  cox <- tryCatch(EnMCB::metricMCB(MCBset = single_res,
                          training_set = related_training,
                          Surv = Surv_training,
                          testing_set = related_testing,
                          Surv.new = Surv_testing,
                          Method = "cox",
                          silent = TRUE),error = function(e){NULL})
  svm<- tryCatch(EnMCB::metricMCB(MCBset = single_res,
                         training_set = related_training,
                         Surv = Surv_training,
                         testing_set = related_testing,
                         Surv.new = Surv_testing,
                         Method = "svm",
                         silent = TRUE),error = function(e){NULL})
  enet<- tryCatch(EnMCB::metricMCB(MCBset = single_res,
                           training_set = related_training,
                           Surv = Surv_training,
                           testing_set = related_testing,
                           Surv.new = Surv_testing,
                           Method = "enet",
                           silent = TRUE),error = function(e){NULL})
  mboost<- tryCatch(EnMCB::metricMCB(MCBset = single_res,
                                   training_set = related_training,
                                   Surv = Surv_training,
                                   testing_set = related_testing,
                                   Surv.new = Surv_testing,
                                   Method = "mboost",
                                   silent = TRUE),error = function(e){NULL})
  data<-rbind(cox$MCB_cox_matrix_training,
              svm$MCB_svm_matrix_training,
              enet$MCB_enet_matrix_training,
              mboost$MCB_mboost_matrix_training
  )
  rownames(data)<-c('cox','svm','enet','mboost')
  data<-t(data)
  data_f<-as.data.frame(data)
  if (ensemble_type == "Standard Cox"){
    univ_models<-tryCatch(rms::cph(formula = Surv_training ~ cox + svm + enet + mboost ,data=data_f),error=function(e){NULL} )
  }else if (ensemble_type == "C-index"){
    #This will let the prediction function use the lambda.1st as the default lambda, which maximum the C-index.
    #It returns the linear model predictions.
    univ_models<-tryCatch(glmnet::cv.glmnet(x=as.matrix(data_f[!is.na(Surv_training),]),
                                            Surv_training[!is.na(Surv_training)],
                                            family='cox',type.measure = 'C'),
                          error=function(e){NULL} )
  }else if (ensemble_type == "feature weighted linear regression"){
  f1<-apply(data_f,1,function(x)e1071::kurtosis(x))
  f2<-apply(data_f,1,function(x)sd(x))
  data_f<-data.frame(f1cox=data_f$cox*f1,f2cox=data_f$cox*f2,
                     f1svm=data_f$svm*f1,f2svm=data_f$svm*f2,
                     f1enet=data_f$enet*f1,f2enet=data_f$enet*f2,
                     f1mboost=data_f$mboost*f1,f2mboost=data_f$mboost*f2)
  
  univ_models<-tryCatch(glmnet::cv.glmnet(x=as.matrix(data_f[!is.na(Surv_training),]),
                                          Surv_training[!is.na(Surv_training)],
                                          family='cox',type.measure = 'C'),
                        error=function(e){NULL} )
  univ_models$ensemble_type = "feature weighted linear regression"
  }else{
    warning("unsupported ensemble type! must be one of Cox and C-index")
    stop(0)
  }
  if (is.null(univ_models)) {
    stop(errorCondition("Ensemble model can't be created, please check your data..."))
  }else{
    res<-list(cox$best_cox_model,
              svm$best_svm_model,
              enet$best_enet_model,
              mboost$best_mboost_model,
              univ_models)
    names(res)<-c("cox","svm","enet","mboost","stacking")
    return(res)
  }
}

#' @title Fast calculation of AUC for ROC using parallel strategy
#'
#' @description This function is used to create time-dependent ROC curve from censored survival data using the Kaplan-Meier (KM) or Nearest Neighbor Estimation (NNE) method of Heagerty, Lumley and Pepe, 2000
#'
#'
#' @param test_matrix Test matrix used in the analysis. Colmuns are samples, rows are markers.
#' @param y_surv Survival information created by Surv function in survival package.
#' @param predict_time Time point of the ROC curve, default is 5 year.
#' @param roc_method Method for fitting joint distribution of (marker,t), either of KM or NNE, the default method is NNE.
#'
#' @author Xin Yu
#' @return
#' This will retrun a numeric vector contains AUC results for each row in test_matrix.
#' @export
#'
#' @examples
#' data(demo_survival_data)
#' data('demo_data',package = "EnMCB")
#' demo_set<-demo_data$realdata
#' res<-fast_roc_calculation(demo_set[1:2,],demo_survival_data)
#' 
fast_roc_calculation<- function(test_matrix,y_surv,predict_time=5,roc_method="NNE") {
  survival_calculation<-function(x,y_surv,predict_time){
    survivalROC::survivalROC(Stime=y_surv[,1],
                             status=y_surv[,2],
                             marker =x,
                             lambda = NULL,
                             predict.time = predict_time,method = "NNE",span =0.25*length(y_surv[,1])^(-0.20))$AUC}
  res <- apply(test_matrix,1, survival_calculation,y_surv,predict_time)
  res
}

#' @title univariate and multivariate survival analysis using coxph
#' @author Xin Yu
#'
#' @param dataframe Clinic data and covariates ready to be tested. Rows are variables and columns are samples.
#' @param y_surv Survival function contain survival data, usually are obtained form Surv() function in survival package.
#' @param digits Integer indicating the number of decimal places.
#' @param asnumeric indicator that the data will be (True) / not (False) transformed into numeric. Default is true.
#' @export
#'
#' @examples
#' data(demo_survival_data)
#' data('demo_data',package = "EnMCB")
#' demo_set<-demo_data$realdata
#' res<-univ_coxph(demo_set,demo_survival_data)
#' @return Object of class \code{matrix} with results.
#' 
univ_coxph <- function(dataframe,y_surv,digits=4,asnumeric=TRUE) {

  #
  rownames_dataframe<-rownames(dataframe)
  res_coxph<-NULL
  for (i in seq_len(nrow(dataframe))) {
    if (asnumeric) {
      covariates <- as.numeric(dataframe[i,])
    }else{
      covariates <- unlist(dataframe[i,])
    }

    univ_models<-survival::coxph(y_surv ~ covariates)
    univ_models<-summary(univ_models)
    p.value<-signif(univ_models$wald["pvalue"], digits)
    wald.test<-signif(univ_models$wald["test"], digits)
    beta<-signif(univ_models$coef[1], digits);#coeficient beta
    HR <-signif(univ_models$coef[2], digits);#exp(beta)
    HR.confint.lower <- signif(univ_models$conf.int[,"lower .95"],digits)
    HR.confint.upper <- signif(univ_models$conf.int[,"upper .95"],digits)
    HR <- paste0(HR, " (",
                 HR.confint.lower, "-", HR.confint.upper, ")")
    res<-c(i,beta, HR, wald.test, p.value)
    names(res)<-c("label","beta", "HR (95% CI for HR)", "wald.test",
                  "p.value")
    res_coxph<-rbind(res_coxph,res)
  }
  colnames(res_coxph)<-c("label","beta", "HR (95% CI for HR)", "wald.test",
                         "p.value")
  res_coxph[,'label']<-rownames_dataframe
  # Source:Benjamini, Y., and Hochberg, Y. (1995).
  # Controlling the false discovery rate: a practical and powerful approach
  # to multiple testing. Journal of the Royal Statistical Society Series B,
  # 57, 289–300. http://www.jstor.org/stable/2346101.
  BH_adjust_p=stats::p.adjust(as.numeric(res_coxph[,"p.value"]),method = "BH")
  # final results
  res_coxph<-cbind(res_coxph,BH_adjust_p)
  res_coxph
}

#'@title Preprocess the Beta value matrix
#'
#'@description This process is optional for the pipeline.
#'This function pre-process the Beta matrix and transform the Beta value into M value.
#'
#'@param met methylation matrix for CpGs. Rows are the CpG names, columns are samples.
#'@param Mvalue Boolean value, TRUE for the M transformation.
#'@param constant_offset the constant offset used in the M transformation formula.
#'@param remove_na Boolean value, if TRUE ,CpGs with NA values will be removed.
#'@param remove_percentage If precentage of NA value exceed the threshold(percentage), the whole CpG probe will be removed. Otherwise, the NA values are replaced with rowmeans.
#'
#'@usage pre_process_methylation(met,Mvalue,constant_offset,remove_na,remove_percentage)
#'
#'@export
#'
#'@return Object of class \code{matrix}.
#'@examples 
#'demo_set<-create_demo()
#'pre_process_methylation(demo_set,Mvalue=FALSE)
#'
pre_process_methylation<-function(met,Mvalue=TRUE,constant_offset=0,remove_na=TRUE,remove_percentage=30){
  threshold=remove_percentage/100
  flag_na<-rep(TRUE,nrow(met))
  names(flag_na)<-rownames(met)
  n<-ncol(met)
  if (remove_na) {
    for (eachrow in rownames(met)) {
      if (sum(is.na(met[eachrow,])|met[eachrow,]==1|met[eachrow,]==0)>n*threshold){
        # whole row will be removed.
        flag_na[eachrow]<-FALSE
      }else{
        if (sum(is.na(met[eachrow,])|met[eachrow,]==1|met[eachrow,]==0)>0){
          met[eachrow,is.na(met[eachrow,])|met[eachrow,]==1|met[eachrow,]==0]<-mean(met[eachrow,(!is.na(met[eachrow,]))&met[eachrow,]!=1&met[eachrow,]!=0])
        }
      }
    }
    met<-met[flag_na,]
  }
  if (Mvalue){
    met<-log2((met+constant_offset)/(1-met+constant_offset))
  }
  met
}

#' @title  Differential expressed methylation correlated blocks
#'
#' @description This function is used to find the Methylation correlated blocks that differentially expressed between groups based on the attractor framework. 
#' This function calculates attractors of all the MCBs among the groups and find the attractor MCBs. \cr
#'
#' @details Currently, only illumina 450k platform is supported. \cr
#' If you want to use other platform, please provide the annotation file with CpG's chromosome and loci. \cr
#' The methylation profile need to convert into matrix format.
#'
#' @param methylation_matrix methylation profile matrix.
#' @param class_vector class vectors that indicated the groups.
#' @param mcb_matrix dataframe or matrix results returned by IdentifyMCB function.
#' @param min.cpgsize threshold for minimum CpGs must included in the individual MCBs.
#' @param pVals_num p value threshold for the test.
#' @param base_method base method used for calculation of differentially methylated regions,
#' should be one of 'Fstat','Tstat','eBayes'. Defualt is Fstat.
#' @param sec_method secondly method in attractor framework, should be one of 'kstest','ttest'. Defualt is ttest.
#' @param ... other parameters pass to the function.
#'
#' @author Xin Yu
#' @return
#' Object of class \code{list} with elements:
#'  \tabular{ll}{
#'    \code{global} \tab Character set contains statistical value for all CpG sites in MCBs. \cr
#'    \code{tab} \tab Matrix contains the information of results. \cr
#'  }
#' @examples
#' data('demo_data', package = "EnMCB")
#' data('demo_survival_data', package = "EnMCB")
#' data('demo_MCBinformation', package = "EnMCB")
#' #Using survival censoring as group label just for demo, 
#' #this may replace with disease and control group in real use.
#' diffMCB_results <- DiffMCB(demo_data$realdata,demo_survival_data[,2], 
#'                            demo_MCBinformation,
#'                            pVals_num = 1)
#'
#' @export
#'
#' @references
#' Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415
#'
#'
DiffMCB<-function(
  methylation_matrix,
  class_vector, 
  mcb_matrix = NULL, 
  min.cpgsize = 5,
  pVals_num = 0.05,
  base_method = c('Fstat','Tstat','eBayes')[1], 
  sec_method = c('ttest','kstest')[1],
  ...
){
  if (is.matrix(methylation_matrix)) {
    dat.fr <- methylation_matrix
  } else {
    warning("The input parameter methylation_matrix must be a matrix. \n
            Now converting it into matrix.\n")
  }
  all.probes <- rownames(dat.fr)
  dat.fr <- as.matrix(dat.fr)
  class.vector <- as.factor(class_vector)
  annotation <- mcb_matrix
  all_included <- strsplit(paste(annotation[,'CpGs'],collapse = ' ')," ")[[1]]
  probes.hits <- intersect(all.probes, all_included)
  dat.detect.w <- dat.fr[rownames(dat.fr) %in% probes.hits,]
  dat.detect.w <- as.matrix(dat.detect.w)
  incidence.matrix <- buildIncidenceMatrix(rownames(dat.detect.w), annotation)
  keep.mcb <- apply(incidence.matrix, 1, sum) >= min.cpgsize
  incidence.matrix <- incidence.matrix[keep.mcb,]
  new.order <- order(class.vector, colnames(dat.detect.w))
  dat.detect.w <- dat.detect.w[, new.order]
  class.vector <- class.vector[new.order]
  
  evalMCB <- function(index, global,sec_method=c('ttest','kstest')[1]) {
    pway.vals <- global[index == 1]
    if (sec_method == 'ttest'){
      res<-t.test(pway.vals, global)
    }else if (sec_method == 'kstest'){
      res<-ks.test(pway.vals, global[index == 0])
    }
    return(c(res$statistic, res$p.value))
  }
  evalp <- function(index, global) {
    return(fisher_combine_p(global[index == 1]))
  }
  
  res<-list()
  if (base_method=='Fstat'){
    fstat <- apply(dat.detect.w, 1, function(y, x) {
      return(anova(lm(y ~ x))[[4]][1])
    }, x = class.vector)
    stat_p <- apply(dat.detect.w, 1, function(y, x) {
      return(anova(lm(y ~ x))[[5]][1])
    }, x = class.vector)
    res$global <- fstat
    stat_p<-apply(incidence.matrix, 1, evalp, global = stat_p)
    fstat <- log(fstat, 2)
    t.pvals <- apply(incidence.matrix, 1, evalMCB, global = fstat,sec_method=sec_method)
  }else if (base_method=='Tstat'){
    tstat <- apply(dat.detect.w, 1, function(y, x) {
      t.test(y ~ x)$statistic
    }, x = class.vector)
    stat_p <- apply(dat.detect.w, 1, function(y, x) {
      t.test(y ~ x)$p.value
    }, x = class.vector)
    res$global <- tstat
    stat_p<-apply(incidence.matrix, 1, evalp, global = stat_p)
    tstat <- log(abs(tstat), 2)
    t.pvals <- apply(incidence.matrix, 1, evalMCB, global = tstat,sec_method=sec_method)
  }else if(base_method=='eBayes'){
    design<-model.matrix(~class.vector)
    fitlim<-limma::lmFit(dat.detect.w,design)
    fiteb<-limma::eBayes(fitlim)
    DEG<-limma::topTable(fiteb,coef=length(unique(class.vector)),n=nrow(dat.detect.w),p.value=1)
    DEG<-DEG[rownames(dat.detect.w),]
    tstat<-DEG$t
    stat_p<-apply(incidence.matrix, 1, evalp, global = DEG$adj.P.Val)
    res$global <- tstat
    tstat <- log(abs(tstat), 2)
    t.pvals <- apply(incidence.matrix, 1, evalMCB, global = tstat, sec_method=sec_method)
 
  }else{
    stop('The method indicator base_method is invalid.')
  }
  if (exists("stat_p")){
    t.pvals_combine <- apply(data.frame(t.pvals[2,],stat_p), 1, fisher_combine_p)
    t.pvals_adjust <- p.adjust(as.numeric(t.pvals_combine), "BH")
  }else{
    t.pvals_adjust <- p.adjust(as.numeric(t.pvals[2,]), "BH")
  }
  ##cat(dim(t.pvals))
  
  size <- apply(incidence.matrix, 1, sum)
  tab <- data.frame(MCBID = rownames(incidence.matrix),
                    annotation[sapply(X = rownames(incidence.matrix),FUN = function(x){strsplit(x, "_")[[1]][2]}),],
                    statistic = as.numeric(t.pvals[1,]),
                    Pvalues = as.numeric(t.pvals[2,]),
                    AdjustedPvalues = t.pvals_adjust, 
                    NumberDetectedCpGs = size)
  #rownames(tab)<-sapply(X = rownames(tab),FUN = function(x){strsplit(x, "_")[[1]][2]})
  tab <- tab[order(t.pvals[2,]), ]
  tab <- subset(tab, AdjustedPvalues<=pVals_num)
  res$tab<-tab
  return(res)
}

#' @title  Identification of methylation correlated blocks
#'
#' @description This function is used to partition the genome into blocks of tightly co-methylated CpG sites, \cr
#' Methylation correlated blocks. This function calculates Pearson correlation coefficients between \cr
#' the beta values of any two CpGs < CorrelationThreshold was used to identify boundaries between any two \cr
#' adjacent markers indicating uncorrelated methylation. Markers not separated by a boundary were combined into MCB. Pearson correlation coefficients between \cr
#' two adjacent CpGs were calculated.
#'
#' @details Currently, only illumina 450k platform is supported, the methylation profile need to convert into matrix format.
#'
#' @param MethylationProfile Methylation matrix is used in the analysis.
#' @param CorrelationThreshold coef correlation threshold is used for define boundaries.
#' @param method method used for calculation of correlation, \cr
#' should be one of "pearson","spearman","kendall". Defualt is "pearson".
#' @param PositionGap CpG Gap between any two CpGs positioned CpG sites less than 1000 bp (default) will be calculated.
#' @param platform This parameter indicates the platform used to produce the methlyation profile. 
#' You can use your own annotation file.
#' @param verbose True as default, which will print the block information for each chromosome. 
#'
#' @author Xin Yu
#' @return
#' Object of class \code{list} with elements:
#'  \tabular{ll}{
#'    \code{MCBsites} \tab Character set contains all CpG sites in MCBs. \cr
#'    \code{MCBinformation} \tab Matrix contains the information of results. \cr
#'  }
#' @examples
#' data('demo_data',package = "EnMCB")
#' 
#' #import the demo TCGA data with 10000+ CpGs site and 455 samples
#' #remove # to run
#' res<-IdentifyMCB(demo_data$realdata)
#' demo_MCBinformation<-res$MCBinformation
#'
#'
#' @export
#'
#' @references
#' Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415
#'
IdentifyMCB<-function(
  MethylationProfile,
  method=c("pearson","spearman","kendall")[1],
  CorrelationThreshold = 0.8,
  PositionGap = 1000,
  platform = "Illumina Methylation 450K",
  verbose = T
){
  if (!(method %in% c("pearson","spearman","kendall"))) {
    stop(paste("Correlation method should be one of pearson, spearman and kendall."))
  }
  cat("Start calculating the correlation, this may take a while...\n")
  FunctionResults<-list()
  if (platform == "Illumina Methylation 450K"){
    Illumina_Infinium_Human_Methylation_450K<-get450kAnno()
    Illumina_Infinium_Human_Methylation_450K<-Illumina_Infinium_Human_Methylation_450K[!is.na(Illumina_Infinium_Human_Methylation_450K[,'pos']),]
    
    intersect_cpg<-intersect(rownames(Illumina_Infinium_Human_Methylation_450K),rownames(MethylationProfile))
    
    met_cg_allgene<-Illumina_Infinium_Human_Methylation_450K[intersect_cpg,]
  }else{
    met_cg_allgene = platform
  }

  MethylationProfile<-MethylationProfile[intersect_cpg,]
  
  chromosomes<-unique(met_cg_allgene[,'chr'])
  chromosomes<-chromosomes[order(chromosomes)]
  res=NULL
  correlation_res<-NULL
  cat("(or you can try to use IdentifyMCB_parallel function instead)\n")
  if (length(chromosomes)>1) {
    bar<-utils::txtProgressBar(min = 1,max = length(chromosomes),char = "#",style = 3)
  }
  for(chr_no in seq_along(chromosomes)){
    if (length(chromosomes)>1) {
      utils::setTxtProgressBar(bar, chr_no)
    }
    chr_id<-chromosomes[chr_no]
    met_x<-MethylationProfile
    if (sum(met_cg_allgene[,'chr'] %in% chr_id)<=2) {
      next
    }
    met_matrix<-met_x[met_cg_allgene[,'chr'] %in% chr_id,]
    ann_matrix<-met_cg_allgene[met_cg_allgene[,'chr'] %in% chr_id,]
    met_matrix<-met_matrix[order(as.numeric(ann_matrix[,'pos']),decreasing = FALSE),]
    ann_matrix<-ann_matrix[order(as.numeric(ann_matrix[,'pos']),decreasing = FALSE),]
    res<-NULL
    total<-nrow(met_matrix)
    for (i in seq_len(total)) {
      # To investigate whether this indeed is evident in our data, we calculated Pearson
      # correlation coefficients between beta values of any two CpGs positioned within
      # one kilobase (or indicated by PositionGap) of one another
      if (i+1<=total){
        if(as.numeric(ann_matrix[i+1,'pos'])-as.numeric(ann_matrix[i,'pos'])<PositionGap &
           ann_matrix[i+1,'chr']==ann_matrix[i,'chr']){
          res<-rbind(res,c(unlist(stats::cor.test(met_matrix[i,],met_matrix[i+1,],method = method))[1:5]))
        }else{
          if (method == "pearson") res<-rbind(res,matrix(c(0,0,1,0,0),1,5))
          else if (method == "spearman") res<-rbind(res,matrix(c(0,1,0,0,0),1,5))
          else if (method == "kendall") res<-rbind(res,matrix(c(0,1,0,0,0),1,5))
        }
      }else{
        if (method == "pearson") res<-rbind(res,matrix(c(0,0,1,0,0),1,5))
        else if (method == "spearman") res<-rbind(res,matrix(c(0,1,0,0,0),1,5))
        else if (method == "kendall") res<-rbind(res,matrix(c(0,1,0,0,0),1,5))
      }
    }
    rownames(res)<-rownames(met_matrix)
    correlation_res<-rbind(correlation_res,res)
  }
  if (verbose) {
    cat("\n")
    cat("Now gathering the results, please wait ...\n")
  }
  met_cg_allgene<-met_cg_allgene[rownames(correlation_res),]
  MCB_flag<-rep("boundary",nrow(correlation_res))
  if (method == "pearson") MCB_flag[as.numeric(correlation_res[,'estimate.cor'])>CorrelationThreshold]<-"MCB"
  else if (method == "spearman") MCB_flag[as.numeric(correlation_res[,'estimate.rho'])>CorrelationThreshold]<-"MCB"
  else if (method == "kendall") MCB_flag[as.numeric(correlation_res[,'estimate.tau'])>CorrelationThreshold]<-"MCB"
  # This CpGs and next one are included.
  MCBsites<-union(grep("MCB",MCB_flag),grep("MCB",MCB_flag)+1)
  MCBsites<-MCBsites[order(MCBsites)]
  FunctionResults$MCBsites<-rownames(MethylationProfile)[MCBsites]

  MCB<-rep(NA,times=10)
  names(MCB)<-c("MCB_no","start","end","CpGs","location","chromosomes",
                "length","MCB_Gene","Feature_Type","CGI_Coordinate")
  MCB_block=FALSE
  MCB_no=1
  total_res<-NULL
  for (i in seq_along(MCB_flag)) {
    flag<-MCB_flag[i]
    if (MCB_block==FALSE&flag=="MCB"){
      MCB["start"] <- i
      MCB["MCB_no"] <- MCB_no
      MCB_no=MCB_no+1
      MCB_block = TRUE
    }
    if (MCB_block==TRUE&flag=="boundary"){
      MCB["end"] <- i
      CpG_names<-rownames(correlation_res)[as.numeric(MCB["start"]):(as.numeric(MCB["end"]))]
      MCB["CpGs"]<- paste(CpG_names,collapse = " ")
      MCB["location"]<-paste(met_cg_allgene[as.numeric(MCB["start"]),'chr'],":",
                                   met_cg_allgene[as.numeric(MCB["start"]),'pos'],"-",
                                   met_cg_allgene[as.numeric(MCB["end"]),'chr'],":",
                                   met_cg_allgene[as.numeric(MCB["end"]),'pos'],collapse = "")
      MCB["chromosomes"]<-met_cg_allgene[as.numeric(MCB["start"]),'chr']
      MCB["MCB_Gene"]<-paste(unique(strsplit(paste(met_cg_allgene[CpG_names,'UCSC_RefGene_Name'],collapse = ";"),";")[[1]]),
                               collapse = " ")
      MCB["Feature_Type"]<-paste(unique(strsplit(paste(met_cg_allgene[CpG_names,'Relation_to_Island'],collapse = ";"),";")[[1]]),
                                   collapse = " ")
      MCB["CGI_Coordinate"]<-paste(unique(met_cg_allgene[CpG_names,'Islands_Name']),
                                     collapse = ";")
      MCB["length"]<-as.numeric(met_cg_allgene[as.numeric(MCB["end"]),'pos'])-
        as.numeric(met_cg_allgene[as.numeric(MCB["start"]),'pos'])
      total_res<-rbind(total_res,MCB)
      MCB_block = FALSE
    }
  }
  total_res<-cbind(total_res,CpGs_num=as.numeric(total_res[,'end'])-as.numeric(total_res[,'start'])+1)
  rownames(total_res)<-total_res[,'MCB_no']
  #Some of 'cgxxxxx' code point to same CpG, those cg-code are removed.
  total_res<-total_res[as.numeric(total_res[,'length'])>1,]
  if (verbose) {
    cat("Statistics (",nrow(total_res)," MCBs in total):\n")
    for (chr_set in unique(total_res[,'chromosomes'])) {
      cat(chr_set,": ")
      cat("total MCBs:",length(total_res[total_res[,'chromosomes'] %in% chr_set,'chromosomes'])," ")
      cat("Mean Length:",mean(as.numeric(total_res[total_res[,'chromosomes'] %in% chr_set,'length']))," ")
      cat("(Range: ",range(as.numeric(total_res[total_res[,'chromosomes'] %in% chr_set,'length'])),")\n")
    }
  }
  FunctionResults$MCBinformation<-total_res
  return(FunctionResults)
}

context("test-hello")

test_that("multiplication works", {
  data("demo_data")
  res<-IdentifyMCB(demo_data$realdata)
  expect_equal(as.character(res$MCBinformation[,'location']), 
   c("chr2 :  74669349 - chr2 :  74669516",
     "chr6 : 108444791 - chr6 : 108444920",
     "chr8 :  18541446 - chr8 :  18541627",
     "chr8 :  61777711 - chr8 :  61778137"))
})

#' Expression matrix of demo dataset.
#' @description 
#' A Expression matrix containing the 10020 CpGs beta value of 455 samples in TCGA lung Adenocarcinoma dataset.
#' This will call from create_demo() function.
#' @format ExpressionSet:
#' \describe{
#'   \item{rownames}{rownames of 10020 CpG features}
#'   \item{colnames}{colnames of 455 samples}
#'   \item{realdata}{Real data matrix for demo.}
#' }
#' @usage data(demo_data)
"demo_data"


#' Survival data of demo dataset.
#' @description
#' A Surv containing survival value of 455 samples in TCGA lung Adenocarcinoma dataset.
#'
#' @format Surv data created by Surv() function in survival package.
#' This data have two unnamed arguments, they will match time and event.
#' @usage data(demo_survival_data)
"demo_survival_data"

#' IlluminaHumanMethylation450kanno
#' @format IlluminaHumanMethylation450kanno.ilmn12.hg19 annotation file.
#' This data have several columns
#' @usage data(anno_matrix)
"anno_matrix"




#' MCB information.
#' @description
#' A dataset containing the number and other attributes of 94 MCBs; 
#' This results was created by the identification function IdentifyMCB.
#' This data used for metricMCB function. 
#'
#' @format A data frame with 94 rows and 8 variables:
#' \describe{
#'   \item{MCB_no}{MCB code}
#'   \item{start}{Start point of this MCB in the chromosome.}
#'   \item{end}{End point of this MCB in the chromosome.}
#'   \item{CpGs}{All the CpGs probe names in the MCB.}
#'   \item{location}{Start, end point and the chromosome number of this MCB.}
#'   \item{chromosomes}{the chromosome number of this MCB.}
#'   \item{length}{the length of bps of this MCB in the chromosome.}
#'   \item{CpGs_num}{number of CpG probes of this MCB.}
#' }
#' @usage data(demo_MCBinformation)
"demo_MCBinformation"

# load private functions
# get450kAnno = function() {
#   
#   ca = BiocFileCache::BiocFileCache()
#   
#   q = BiocFileCache::bfcquery(ca, "450kanno.ilmn12.hg19")
#   
#   if (length(q$rpath)>0) return(readRDS(rev(q$rpath)[1])) # if multiple, use last
#   
#   if (!requireNamespace("minfi")) stop("install the minfi package to use this function")
#   
#   anno = minfi::getAnnotation("IlluminaHumanMethylation450kanno.ilmn12.hg19")
#   
#   tf = tempfile()
#   
#   saveRDS(anno, tf)
#   
#   BiocFileCache::bfcadd(ca, rname="IlluminaHumanMethylation450kanno.ilmn12.hg19", fpath=tf,
#                         action="move")  # for future
#   anno
# }

get450kAnno = function() {
  utils::data(anno_matrix,package = "EnMCB")
  anno_matrix
}

print_as_data <- function(variables,file) {
  sink(file)
  print(variables)
  sink()
}

ridge_model <- function(times,data_used_for_training,theta = 1){
  survival::coxph(times ~ survival::ridge(allvars,theta = theta),data=data_used_for_training)
}

#' @title ridge matrix
#' @description as.matrix attempts to turn its argument 
#' @param x data vector
#' @usage as.ridgemat(x)
#' @export
as.ridgemat <- function(x) {
  class(x) <- c("ridgemat", class(x))
  x
}

as.mcb.coxph.penal <- function(x){
  class(x) <- c("mcb.coxph.penal", class(x))
  x
}

removeobjectclass<- function(x){
  class(x) <- setdiff(class(x),"mcb.coxph.penal")
  x
}

#' @title data frame ridge matrix
#' @description data frame ridge matrix
#' @param x data vector
#' @param ... other parameters pass to as.data.frame.model.matrix()
#' @method as.data.frame ridgemat
#' @export
as.data.frame.ridgemat <- function(x, ...) {
  as.data.frame.model.matrix(as.matrix(x), ...)
}


#'@title predict coxph penal using MCB
#'@author Xin Yu
#'@description Compute fitted values and regression terms for a model fitted by coxph
#'@param object the results of a coxph fit.
#'@param newdata Optional new data at which to do predictions. If absent predictions are for the data frame used in the original fit. 
#'When coxph has been called with a formula argument created in another context, i.e., coxph has been called within another function and the formula was passed as an argument to that function, 
#'there can be problems finding the data set. See the note below.
#'@param ... other parameters pass to predict.coxph
#'@method predict mcb.coxph.penal
#'@return prediction values of regression.
#'@export
predict.mcb.coxph.penal <- function(object,newdata,...){
  internames = colnames(newdata) %in% object$CpGs
  if (sum(internames) == length(object$CpGs)){
    newdata <- data.frame(allvars = as.ridgemat(newdata[,internames]))
    object = removeobjectclass(object)
    return(predict(object,newdata,...))
  }else{
    stop('newdata do not contain sufficient CpGs in the model')
  }
}


test_logrank_p_in_KM<-function(mcb_matrix,y_surv){
  p.val_all<-NULL
  for (mcb in rownames(mcb_matrix)){
    Test<-mcb_matrix[mcb,]
    group_sur<-factor(1*(Test>median(Test)),
                      levels=0:1,
                      labels=c("High risk","Low risk"))
    diff_tcga<-survival::survdiff(y_surv ~ group_sur)
    p.val <- 1 - stats::pchisq(diff_tcga$chisq, length(diff_tcga$n) - 1)
    p.val_all<-rbind(p.val_all,p.val)
  }
  names(p.val_all)<-rownames(mcb_matrix)
  p.val_all
}


give_me_the_sen_spc<-function(pred,print=TRUE) {
  tpr<-(pred@tp[[1]]/(pred@tp[[1]]+pred@fn[[1]])) #sensitivity
  fpr<-(pred@fp[[1]]/(pred@tn[[1]]+pred@fp[[1]])) #specificity
  best_one<-which.min(sqrt( (1-tpr)^2+ fpr^2 ))
  if (print == TRUE)  cat(paste(" sensitivity:",tpr[best_one],"\n","specificity:",1-fpr[best_one],"\n"))
  pred@cutoffs[[1]][best_one]#specificity
}

random_matrix <- function(x) {
  x_random=matrix(x[sample(seq_along(x),length(x))],ncol = ncol(x),nrow = nrow(x))
  rownames(x_random)<-rownames(x)
  colnames(x_random)<-colnames(x)
  x_random
}


#'@title create demo matrix
#'@author Xin Yu
#'@description Demo matrix for methylation matrix.
#'@param model Two options, 'all' or 'short' for creating full dataset or very brief demo.
#'@return This function will generate a demo data.
#'@export
#'@examples 
#'demo_set<-create_demo()
create_demo<-function(model=c('all','short')[1]){
  utils::data(demo_data)
  rmax<-matrix(data = rnorm(length(demo_data$rownames)*length(demo_data$colnames), mean=0.5, sd=0.08),
           nrow = length(demo_data$rownames),
           ncol=length(demo_data$colnames),dimnames = list(demo_data$rownames,demo_data$colnames)
           )
  rmax[rownames(demo_data$realdata),]<-demo_data$realdata
  if (model=='short') {
    rmax<-rmax[ceiling(nrow(rmax)*0.8):nrow(rmax),]
  }
  rmax
}



as.numeric_matrix<-function(met_matrixf){
  met_matrixff<-matrix(as.numeric(met_matrixf),
                       nrow = NROW(met_matrixf),
                       ncol = NCOL(met_matrixf))
  rownames(met_matrixff)<-rownames(met_matrixf)
  colnames(met_matrixff)<-colnames(met_matrixf)
  met_matrixff
}

makeEset<-function(met_matrix,cli_data){
  all_col<-NULL
  for (i in seq_len(ncol(met_matrix))) {
    if (length(grep(pattern=tolower(colnames(met_matrix)[i]),cli_data))>0) {
      loca<-grep(pattern=tolower(colnames(met_matrix)[i]),cli_data)[1]
      col<-loca%/%nrow(cli_data)+1
      row<-loca%%nrow(cli_data)
      all_col<-c(all_col,col)
    }
  }
  cli_data_f<-cli_data[,all_col]
  colnames(cli_data_f)<-colnames(met_matrix)
  eSet<-make_eset(met_matrix,data.frame(t(cli_data_f)))
  eSet
}


multiple_time_ROC <- function(Test,y_surv,genesel) {
  #requireNamespace("prognosticROC") #version 0.7
  sroclong_all<-NULL
  for (n_time in c(3,5,7)) {
    ROC_res= survivalROC::survivalROC(Stime=y_surv[,1],
                         status=y_surv[,2],
                         marker =as.numeric(Test),
                         lambda = NULL,window = "asymmetric",
                         predict.time = n_time,method = "NNE",span =0.25*length(y_surv[,1])^(-0.20))#
    # AUC at 7 years: 0·670 (0·578–0·762)
    sroclong_all<-ROCdata_save(sroclong_all,ROC_res,mark = paste("AUC at",n_time,"years:",round(ROC_res$AUC,2),collapse = " "))
  }
  gg<-ggplot(sroclong_all, aes(x = FPF, y = TPF, label = c, color = groups))+
    coord_cartesian(ylim=c(0, 1.05),xlim=c(0, 1.05),expand=FALSE)+
    geom_roc(labels = FALSE, stat = "identity") + style_roc(theme = theme_light())+
    theme(panel.grid.major =element_blank(), panel.grid.minor = element_blank(),
          panel.background = element_blank(),axis.line = element_line(colour = "black"))+
    theme(legend.position=c(0.8,0.35), legend.justification=c(1,1),legend.title = element_blank())+
    geom_abline(slope=1, colour="black")
  ggsave(filename = paste("Time ROC of ",genesel,".jpeg",sep=""),plot = gg,device ="jpeg" ,
         path = getwd(),dpi = 300,units = "in",width = 5, height = 4.5,
         limitsize=FALSE)
}

draw_pheatmap_matrix<-function(eSet_matrix,group,savename="draw_pheatmap_matrix",scale = FALSE){
  requireNamespace(pheatmap)
  requireNamespace(ggplot2)
  group<-as.character(group)
  df<-as.numeric_matrix(eSet_matrix)
  df<-df[,order(group,decreasing = TRUE)]
  group<-group[order(group,decreasing = TRUE)]
  df[is.na(df)]<-0
  aka2 = data.frame(ID = group)
  rownames(aka2)<-colnames(df)
  aka3 = list(ID = c('TRUE' = "blue", 'FALSE'="red"))

  df<-df[apply(df, 1,stats::sd)>0,]
  if (scale == TRUE) {
    df=scale(df)
  }
   gg<-pheatmap(df,
           annotation_col = aka2,
           annotation_colors = aka3[1],
           annotation_legend = FALSE,
           show_colnames = FALSE, show_rownames = FALSE, cluster_rows = TRUE,
           cluster_cols = FALSE, legend = TRUE,
           clustering_distance_rows = "euclidean", border_color = FALSE)
   ggplot2::ggsave(filename = paste(savename,".jpeg",sep=""),plot = gg,device ="jpeg" ,
          path = getwd(),dpi = 300,units = "in",width = 10, height = 5,
          limitsize=FALSE)
}


give_me_the_lasso_gene<-function(cvfit,s=NULL,plot_figure=TRUE){
  if (plot_figure ==TRUE) {
    plot(cvfit)
    # cat("min: ",cvfit$lambda.min,"\n")
    # cat("1se: ",cvfit$lambda.1se,"\n")
  }
  if (is.null(s)) {
    s=cvfit$lambda.min
  }
  Coefficients <- coef(cvfit, s = s) #0.20
  Active.Index <- which(as.logical(Coefficients != 0))
  Active.Coefficients <- Coefficients[Active.Index]
  #print(Active.Coefficients)
  gene_lasso<-Coefficients@Dimnames[[1]][Active.Index]
  if (plot_figure ==TRUE) {
    plot(cvfit$glmnet.fit,xvar="lambda")
  }
  list(gene_lasso=gene_lasso,Active.Coefficients=Active.Coefficients)
}


plot_ROC<-function(perf){
  sroclong<-data.frame(TPF = perf@y.values[[1]], FPF =perf@x.values[[1]],
                       c = perf@alpha.values[[1]] #,
                       #group = rep("M stage", length(Mayo5[["FP"]]))
  )
  ggplot(sroclong, aes(x = FPF, y = TPF, label = c
                       #, color = group
  )) +
  geom_roc(labels = FALSE, stat = "identity") + style_roc(theme = theme_grey)
}

ROC_mutiple_clinical<-function(test_frame,y_surv,genesel="title",ntime=5){
    #requireNamespace("prognosticROC") #version 0.7
    sroclong_all<-NULL
    for (n in seq_len(ncol(test_frame))) {
      ROC_res= survivalROC::survivalROC(Stime=y_surv[,1],
                                        status=y_surv[,2],
                                        marker =as.numeric(test_frame[,n]),
                                        lambda = NULL,window = "asymmetric",
                                        predict.time = ntime,method = "NNE",span =0.25*length(y_surv[,1])^(-0.20))#
      sroclong_all<-ROCdata_save(sroclong_all,ROC_res,mark = paste(ntime,"year AUC at",colnames(test_frame)[n],round(ROC_res$AUC,2),collapse = " "))
    }
    gg<-ggplot2::ggplot(sroclong_all, aes(x = FPF, y = TPF, label = c, color = groups))+
      coord_cartesian(ylim=c(0, 1.05),xlim=c(0, 1.05),expand=FALSE)+
      geom_roc(labels = FALSE, stat = "identity") + style_roc(theme = theme_light())+
      theme(panel.grid.major =element_blank(), panel.grid.minor = element_blank(),
            panel.background = element_blank(),axis.line = element_line(colour = "black"))+
      theme(legend.position=c(0.8,0.35), legend.justification=c(1,1),legend.title = element_blank())+
      geom_abline(slope=1, colour="black")
    ggplot2::ggsave(filename = paste("Time ROC of ",genesel,".jpeg",sep=""),plot = gg,device ="jpeg" ,
           path = getwd(),dpi = 300,units = "in",width = 5, height = 4.5,
           limitsize=FALSE)
  }

ROC_threshold <- function(predict, response) {
  perf <-prediction(as.numeric(predict),as.numeric(response))
  return(give_me_the_sen_spc(perf,print = FALSE))
}

ROCdata_save<-function(origin=NULL,perf,mark="none"){
  sroclong<-data.frame(TPF = perf[["TP"]], FPF = perf[["FP"]],
                       c = perf[["cut.values"]],
                       groups = rep(mark,
                                    length(perf[["FP"]])
                                    )
                       )
  sroclong<-rbind(origin,sroclong)
  sroclong
}

mutiple_time_ROC <- function(Test,y_surv,genesel) {
  requireNamespace(survivalROC) #version 1.0.3
  #requireNamespace(prognosticROC) #version 0.7
  sroclong_all<-NULL
  for (n_time in c(3,5,7)) {
    ROC_res= survivalROC::survivalROC(Stime=y_surv[,1],
                         status=y_surv[,2],
                         marker =as.numeric(Test),
                         lambda = NULL,window = "asymmetric",
                         predict.time = n_time,method = "NNE",span =0.25*length(y_surv[,1])^(-0.20))#
    # AUC at 7 years: 0·670 (0·578–0·762)

    sroclong<-data.frame(TPF = ROC_res[["TP"]], FPF = ROC_res[["FP"]],
                         c = ROC_res[["cut.values"]],
                         groups = rep(paste("AUC at",n_time,"years:",round(ROC_res$AUC,2),collapse = " "),
                                     length(ROC_res[["FP"]])))
    sroclong_all<-rbind(sroclong_all,sroclong)
  }
  gg<-ggplot(sroclong_all, aes(x = FPF, y = TPF, label = c, color = groups))+
    coord_cartesian(ylim=c(0, 1.05),xlim=c(0, 1.05),expand=FALSE)+
    geom_roc(labels = FALSE, stat = "identity") + style_roc(theme = theme_light())+
    theme(panel.grid.major =element_blank(), panel.grid.minor = element_blank(),
          panel.background = element_blank(),axis.line = element_line(colour = "black"))+
    theme(legend.position=c(0.8,0.35), legend.justification=c(1,1),legend.title = element_blank())+
    geom_abline(slope=1, colour="black")
  ggsave(filename = paste("Time ROC of ",genesel,".jpeg",sep=""),plot = gg,device ="jpeg" ,
         path = getwd(),dpi = 300,units = "in",width = 5, height = 4.5,
         limitsize=FALSE)
}




#'@title draw survival curve
#'@author Xin Yu
#'@description Draw a survival curve based on survminer package. This is a wrapper function of ggsurvplot.
#'@param exp expression level for variable.
#'@param living_days The survival time (days) for each individual.
#'@param living_events The survival event for each individual, 0 indicates alive and 1 indicates death.
#'Other choices are TRUE/FALSE (TRUE = death) or 1/2 (2=death). For interval censored data, the status indicator is 0=right censored, 1=event at time, 2=left censored, 3=interval censored.
#'@param write_name The name for pdf file which contains the result figure.
#'@param title_name The title for the result figure.
#'@param threshold Threshold used to indicate the high risk or low risk.
#'@param file If True, function will automatic generate a result pdf, otherwise it will
#'return a ggplot object. Default is FALSE.
#'
#'@return This function will generate a pdf file with 300dpi which compare survival curves using the Kaplan-Meier (KM) test.
#'@export
#'@examples 
#' data(demo_survival_data)
#' library(survival)
#' demo_set<-create_demo()
#' draw_survival_curve(demo_set[1,],
#'     living_days = demo_survival_data[,1],
#'     living_events =demo_survival_data[,2],
#'     write_name = "demo_data" )
#'
draw_survival_curve<-function(exp,living_days,living_events,write_name,title_name="",threshold=NA,file = FALSE){
  if (is.na(threshold)) threshold=median(exp)
  group_sur<-factor(1*(exp >threshold) ,
                    levels=0:1,
                    labels=c("low risk", "high risk"))
  pdata_gene <- list(time=living_days,
                     event=living_events,
                     gene=exp,
                     group_sur=group_sur)
  fit <- survival::survfit(survival::Surv(time, event) ~ group_sur, data = pdata_gene)
  data.survdiff <- survival::survdiff(survival::Surv(time, event) ~ group_sur, data = pdata_gene)
  p.val = 1 - stats::pchisq(data.survdiff$chisq, length(data.survdiff$n) - 1)
  if (p.val<0.00001){
    p_value="p value < 0.00001"
  }else{
    p_value=paste("p =",round(p.val,5),collapse = " ")
  }
  HR = (data.survdiff$obs[2]/data.survdiff$exp[2])/(data.survdiff$obs[1]/data.survdiff$exp[1])
  up95 = exp(log(HR) + qnorm(0.975)*sqrt(1/data.survdiff$exp[2]+1/data.survdiff$exp[1]))
  low95 = exp(log(HR) - qnorm(0.975)*sqrt(1/data.survdiff$exp[2]+1/data.survdiff$exp[1]))
  #require(ggfortify)
  gg<-survminer::ggsurvplot(fit,data = pdata_gene,  risk.table = TRUE,risk.table.y.text = TRUE,break.time.by=1,
                            fillLineSize=3,survSize=5,surv.linesize=5,pval = FALSE,
                            fontsize=2.5,
                            legend.labs = c("low risk","high risk"),xlab="Time (years)",
                        censor.alpha=0.5,palette=c("#00BFC4","#F8766D"),
                        title=paste(title_name ,"\n",p_value," HR ",round(HR,2),
                                         "(",round(up95,2) ,"-", round(low95,2),  ")",collapse = ""),
  ggtheme=ggplot2::theme(title =ggplot2::element_text(size=8, face='bold'),
          panel.grid.major =ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank(),
    panel.background = ggplot2::element_blank(),axis.line = ggplot2::element_line(colour = "black"),
    legend.position=c(1,1), legend.justification=c(1,1),
    legend.background = ggplot2::element_rect(fill = 'white', colour = 'white'))
  )
  if (file){
    ggplot2::ggsave(filename = paste("survival of ",write_name,".jpeg",sep=""),plot = print(gg),device ="jpeg" ,
                    path = getwd(),dpi = 300,units = "in",width = 5, height = 5,
                    limitsize=FALSE)
  }else{
    return(gg)
  }

}


is.integer0 <- function(x)
{
  is.integer(x) && length(x) == 0L
}

bs_ci <- function(data, indices, predict.time) { 
  d <- data[indices,] # allows boot to select sample
  d <- d[!is.na(d$survival),]
  surv.res = survivalROC::survivalROC.C(Stime = d$survival, 
                         status = d$survival_status, 
                         marker = d$marker, 
                         predict.time,
                         span = 0.25*NROW(d)^(-0.20))
  return(surv.res$AUC)
}

calculate_auc_ci <- function(survival, marker, predict_time,ci){
  if (ci){
    ci_res = boot::boot(data=data.frame(survival = survival[,1],
                                    survival_status = survival[,2],
                                    marker = marker),
                    statistic=bs_ci, R=1000, predict.time = predict_time)
    res = boot::boot.ci(ci_res,type="perc")
    return(list( AUC = res$t0, 
                 CI95 = paste(format(res$percent[,4], digits = 4),"-", format(res$percent[,5], digits = 4))
    ))
  }else{
    res = survivalROC::survivalROC.C(Stime = survival[,1], 
                               status = survival[,2], 
                               marker = marker, 
                               predict.time = predict_time,
                               span = 0.25*length(survival)^(-0.20))
    return(res)
  }

}

metricMCB.mean<-function(MCBset,MCB_matrix,Surv,data_set,show_bar=T){
  FunctionResults<-list()
  MCB_model_res<-NULL
  if (show_bar) {
    bar<-utils::txtProgressBar(min = 1,max = nrow(MCBset),char = "#",style = 3)
  }
  for (mcb in seq(nrow(MCBset))) {
    utils::setTxtProgressBar(bar, mcb)
    write_MCB<-rep(NA,5)
    #save the mcb number
    write_MCB[1]<-as.numeric(MCBset[mcb,'MCB_no'])
    write_MCB[2]<-MCBset[mcb,'CpGs']
    CpGs<-strsplit(MCBset[mcb,'CpGs']," ")[[1]]
    MCB_matrix[mcb,]<-colMeans(data_set[CpGs,])
    AUC_value<-survivalROC::survivalROC(Stime = Surv[,1],
                                        status = Surv[,2],
                                        marker = MCB_matrix[mcb,],
                                        predict.time = 5,
                                        method = "NNE",
                                        span =0.25*length(Surv)^(-0.20))$AUC
    write_MCB[3]<-AUC_value
    reg<-survival::survreg(Surv ~ MCB_matrix[mcb,])
    cindex<-survival::concordance(reg)
    write_MCB[4]<-cindex$concordance
    write_MCB[5]<-cindex$cvar
    MCB_model_res<-rbind(MCB_model_res,write_MCB)
  }
  colnames(MCB_model_res)<-c("MCB_no","CpGs","auc","C-index","C-index_SE")
  rownames(MCB_matrix)<-MCB_model_res[,'MCB_no']
  FunctionResults$MCB_matrix<-MCB_matrix
  FunctionResults$auc_results<-MCB_model_res
  return(FunctionResults)
}


ensemble_prediction.m<- function(ensemble_model,prediction_data) {
  prediction_data <- prediction_data[ensemble_model$cox$cox_model$CpGs,]
  if (nrow(prediction_data) != length(rownames(prediction_data))) {
    stop("ERROR: The predition data and the model have wrong dimensions!")
  }
  rank_svm <- stats::predict(ensemble_model$svm$svm_model, data.frame(t(prediction_data)))$predicted[1,]
  svm <- predict(ensemble_model$svm$hr_model,data.frame(rank_svm = rank_svm),type='lp')
  cox <-stats::predict(ensemble_model$cox$cox_model, data.frame(t(prediction_data)))
  enet <- stats::predict(ensemble_model$enet$enet_model, t(prediction_data), 
                         s = ensemble_model$enet$`corrected_lambda(min)`)
  mboost<-stats::predict(ensemble_model$mboost$mboost_model, t(prediction_data))[,1]
  data<-rbind(cox,
              svm,
              t(enet),
              mboost
  )
  rownames(data)<-c('cox','svm','enet','mboost')
  data<-t(data)
  data_f<-data.frame(cox = cox,svm=rank_svm,enet=as.numeric(enet),mboost = mboost)
  if (class(ensemble_model$stacking)[1] == "cv.glmnet")
    ensemble = as.numeric(stats::predict(ensemble_model$stacking, as.matrix(data_f)))
  else if (class(ensemble_model$stacking)[1] == "cph")
    ensemble = stats::predict(ensemble_model$stacking, data_f,type='lp')
  else stop("ERROR: The ensemble predition model is invaild !")
  return(t(cbind(data,ensemble)))
}

#inherent from survivalROC package to calculate ROC with small modification
survivalROC_C <- function (Stime, status, marker, entry = NULL, predict.time, 
                         cut.values = NULL, method = "NNE", lambda = NULL, span = NULL, 
                         window = "symmetric") 
{
  times = Stime
  x <- marker
  if (is.null(entry)) 
    entry <- rep(0, length(times))
  bad <- is.na(times) | is.na(status) | is.na(x) | is.na(entry)
  entry <- entry[!bad]
  times <- times[!bad]
  status <- status[!bad]
  x <- x[!bad]
  if (sum(bad) > 0) 
    cat(paste("\n", sum(bad), "records with missing values dropped. \n"))
  if (is.null(cut.values)) 
    cut.values <- unique(x)
  cut.values <- cut.values[order(cut.values)]
  ncuts <- length(cut.values)
  ooo <- order(times)
  times <- times[ooo]
  status <- status[ooo]
  x <- x[ooo]
  s0 <- 1
  unique.t0 <- unique(times)
  unique.t0 <- unique.t0[order(unique.t0)]
  n.times <- sum(unique.t0 <= predict.time)
  for (j in 1:n.times) {
    n <- sum(entry <= unique.t0[j] & times >= unique.t0[j])
    d <- sum((entry <= unique.t0[j]) & (times == unique.t0[j]) & 
               (status == 1))
    if (n > 0) 
      s0 <- s0 * (1 - d/n)
  }
  s.pooled <- s0
  roc.matrix <- matrix(NA, ncuts, 2)
  roc.matrix[ncuts, 1] <- 0
  roc.matrix[ncuts, 2] <- 1
  if (method == "KM") {
    for (c in 1:(ncuts - 1)) {
      s0 <- 1
      subset <- as.logical(x > cut.values[c])
      e0 <- entry[subset]
      t0 <- times[subset]
      c0 <- status[subset]
      if (!is.null(t0)) {
        unique.t0 <- unique(t0)
        unique.t0 <- unique.t0[order(unique.t0)]
        n.times <- sum(unique.t0 <= predict.time)
        if (n.times > 0) {
          for (j in 1:n.times) {
            n <- sum(e0 <= unique.t0[j] & t0 >= unique.t0[j])
            d <- sum((e0 <= unique.t0[j]) & (t0 == unique.t0[j]) & 
                       (c0 == 1))
            if (n > 0) 
              s0 <- s0 * (1 - d/n)
          }
        }
      }
      p0 <- mean(subset)
      roc.matrix[c, 1] <- (1 - s0) * p0/(1 - s.pooled)
      roc.matrix[c, 2] <- 1 - s0 * p0/s.pooled
    }
  }
  if (method == "NNE") {
    if (is.null(lambda) & is.null(span)) {
      cat("method = NNE requires either lambda or span! \n")
      stop(0)
    }
    x.unique <- unique(x)
    x.unique <- x.unique[order(x.unique)]
    S.t.x <- rep(0, length(x.unique))
    t.evaluate <- unique(times[status == 1])
    t.evaluate <- t.evaluate[order(t.evaluate)]
    t.evaluate <- t.evaluate[t.evaluate <= predict.time]
    for (j in 1:length(x.unique)) {
      if (!is.null(span)) {
        if (window == "symmetric") {
          ddd <- (x - x.unique[j])
          n <- length(x)
          ddd <- ddd[order(ddd)]
          index0 <- sum(ddd < 0) + 1
          index1 <- index0 + trunc(n * span + 0.5)
          if (index1 > n) 
            index1 <- n
          lambda <- ddd[index1]
          wt <- as.integer(((x - x.unique[j]) <= lambda) & 
                             ((x - x.unique[j]) >= 0))
          index0 <- sum(ddd <= 0)
          index2 <- index0 - trunc(n * span/2)
          if (index2 < 1) 
            index2 <- 1
          lambda <- abs(ddd[index2])
          set.index <- ((x - x.unique[j]) >= -lambda) & 
            ((x - x.unique[j]) <= 0)
          wt[set.index] <- 1
        }
        if (window == "asymmetric") {
          ddd <- (x - x.unique[j])
          n <- length(x)
          ddd <- ddd[order(ddd)]
          index0 <- sum(ddd < 0) + 1
          index <- index0 + trunc(n * span)
          if (index > n) 
            index <- n
          lambda <- ddd[index]
          wt <- as.integer(((x - x.unique[j]) <= lambda) & 
                             ((x - x.unique[j]) >= 0))
        }
      }
      else {
        wt <- exp(-(x - x.unique[j])^2/lambda^2)
      }
      s0 <- 1
      for (k in 1:length(t.evaluate)) {
        n <- sum(wt * (entry <= t.evaluate[k]) & (times >= 
                                                    t.evaluate[k]))
        d <- sum(wt * (entry <= t.evaluate[k]) & (times == 
                                                    t.evaluate[k]) * (status == 1))
        if (n > 0) 
          s0 <- s0 * (1 - d/n)
      }
      S.t.x[j] <- s0
    }
    S.all.x <- S.t.x[match(x, x.unique)]
    n <- length(times)
    S.marginal <- sum(S.all.x)/n
    for (c in 1:(ncuts - 1)) {
      p1 <- sum(x > cut.values[c])/n
      Sx <- sum(S.all.x[x > cut.values[c]])/n
      roc.matrix[c, 1] <- (p1 - Sx)/(1 - S.marginal)
      roc.matrix[c, 2] <- 1 - Sx/S.marginal
    }
  }
  sensitivity = roc.matrix[, 1]
  specificity = roc.matrix[, 2]
  x <- 1 - c(0, specificity)
  y <- c(1, sensitivity)
  n <- length(x)
  dx <- x[-n] - x[-1]
  mid.y <- (y[-n] + y[-1])/2
  area <- sum(dx * mid.y)
  list(cut.values = c(-Inf, cut.values), TP = y, FP = x, predict.time = predict.time, 
       Survival = s.pooled, AUC = area)
}

s_t_x<-function (Stime, status, marker, entry = NULL, predict.time) 
{
  span = 0.25*length(marker)^(-0.20)
  lambda = NULL
  times = Stime
  x <- marker
  if (is.null(entry)) 
    entry <- rep(0, length(times))
  bad <- is.na(times) | is.na(status) | is.na(x) | is.na(entry)
  entry <- entry[!bad]
  times <- times[!bad]
  status <- status[!bad]
  x <- x[!bad]
  if (sum(bad) > 0) 
    cat(paste("\n", sum(bad), "records with missing values dropped. \n"))
  cut.values <- unique(x)
  cut.values <- cut.values[order(cut.values)]
  ncuts <- length(cut.values)
  ooo <- order(times)
  times <- times[ooo]
  status <- status[ooo]
  x <- x[ooo]
  s0 <- 1
  unique.t0 <- unique(times)
  unique.t0 <- unique.t0[order(unique.t0)]
  n.times <- sum(unique.t0 <= predict.time)
  for (j in 1:n.times) {
    n <- sum(entry <= unique.t0[j] & times >= unique.t0[j])
    d <- sum((entry <= unique.t0[j]) & (times == unique.t0[j]) & 
               (status == 1))
    if (n > 0) 
      s0 <- s0 * (1 - d/n)
  }
  s.pooled <- s0
  roc.matrix <- matrix(NA, ncuts, 2)
  roc.matrix[ncuts, 1] <- 0
  roc.matrix[ncuts, 2] <- 1
  x.unique <- unique(x)
  x.unique <- x.unique[order(x.unique)]
  S.t.x <- rep(0, length(x.unique))
  t.evaluate <- unique(times[status == 1])
  t.evaluate <- t.evaluate[order(t.evaluate)]
  t.evaluate <- t.evaluate[t.evaluate <= predict.time]
  for (j in 1:length(x.unique)) {
    if (!is.null(span)) {
      ddd <- (x - x.unique[j])
      n <- length(x)
      ddd <- ddd[order(ddd)]
      index0 <- sum(ddd < 0) + 1
      index1 <- index0 + trunc(n * span + 0.5)
      if (index1 > n) 
        index1 <- n
      lambda <- ddd[index1]
      wt <- as.integer(((x - x.unique[j]) <= lambda) & 
                         ((x - x.unique[j]) >= 0))
      index0 <- sum(ddd <= 0)
      index2 <- index0 - trunc(n * span/2)
      if (index2 < 1) 
        index2 <- 1
      lambda <- abs(ddd[index2])
      set.index <- ((x - x.unique[j]) >= -lambda) & 
        ((x - x.unique[j]) <= 0)
      wt[set.index] <- 1
      
    }
    else {
      cat("requires a valid span! \n")
      stop(0)
    }
    s0 <- 1
    for (k in 1:length(t.evaluate)) {
      n <- sum(wt * (entry <= t.evaluate[k]) & (times >= 
                                                  t.evaluate[k]))
      d <- sum(wt * (entry <= t.evaluate[k]) & (times == 
                                                  t.evaluate[k]) * (status == 1))
      if (n > 0) 
        s0 <- s0 * (1 - d/n)
    }
    S.t.x[j] <- s0
  }
  S.all.x <- S.t.x[match(x, x.unique)]
  ks_res<-ks.test(x=S.all.x,y="exp",alternative = "t")
  list(S.all.x = S.all.x,ks_res = ks_res)
}

#error metrics -- Confusion Matrix
err_metric=function(CM,verbose = F)
{
  TP =CM[1,1]
  TN =CM[2,2]
  FP =CM[2,1]
  FN =CM[1,2]
  res = list()
  res$precision =TP / (TP + FP)
  res$recall_score =TP / (TP + FN)
  res$f1_score=2*((res$precision*res$recall_score)/(res$precision+res$recall_score))
  res$accuracy_model  = (TP+TN) / (TP+FN+FP+TN)
  res$False_positive_rate = (FP)/(FP+TN)
  res$False_negative_rate = (FN)/(FN+TP)
  if (verbose == T) {
    return(res)
  }else{
    print(paste("Precision value of the model: ",round(res$precision,2)))
    print(paste("Accuracy of the model: ",round(res$accuracy_model,2)))
    print(paste("Recall value of the model: ",round(res$recall_score,2)))
    print(paste("False Positive rate of the model: ",round(res$False_positive_rate,2)))
    print(paste("False Negative rate of the model: ",round(res$False_negative_rate,2)))
    print(paste("F1 score of the model: ",round(res$f1_score,2)))
  }
}

prepare_data_frame<-function(df_data){
  final_data<-NULL
  for (i in 1:ncol(df_data)) {
    one_data<-data.frame(samples= 1:nrow(df_data),
                         var_name=rep(colnames(df_data)[i],nrow(df_data)),
                         var_num=df_data[,i])
    final_data<-rbind(final_data,one_data)
  }
  final_data
}

get_the_loci <- function(CpGs,pos = TRUE){
  CpGs_t<-strsplit(CpGs, " ")[[1]]
  Illumina_Infinium_Human_Methylation_450K <- get450kAnno()
  met_cg_allgene <- Illumina_Infinium_Human_Methylation_450K[CpGs_t,]
  if (pos) met_cg_allgene[,'pos'] else met_cg_allgene
}

findAttractors<-function (methylation_matrix, annotation, class_vector, 
                          min.pwaysize = 5,base_method = c('Fstat','Tstat')[1], sec_method = c('ttest','kstest')[1] ,...) 
{
  
  dat.fr <- methylation_matrix
  all.probes <- rownames(dat.fr)
  dat.fr <- as.matrix(dat.fr)
  class.vector <- as.factor(class_vector)
  all_included <- strsplit(paste(annotation[,'CpGs'],collapse = ' ')," ")[[1]]
  probes.hits <- intersect(all.probes, all_included)
  dat.detect.w <- dat.fr[rownames(dat.fr) %in% probes.hits,]
  dat.detect.w <- as.matrix(dat.detect.w)
  incidence.matrix <- buildIncidenceMatrix(rownames(dat.detect.w), annotation)
  keep.mcb <- apply(incidence.matrix, 1, sum) >= min.pwaysize
  incidence.matrix <- incidence.matrix[keep.mcb,]
  new.order <- order(class.vector, colnames(dat.detect.w))
  dat.detect.w <- dat.detect.w[, new.order]
  class.vector <- class.vector[new.order]
  
  evalPway <- function(index, global,sec_method=c('ttest','kstest')[1]) {
    pway.vals <- global[index == 1]
    if (sec_method == 'ttest'){
      return(t.test(pway.vals, global)$p.value)
    }else if (sec_method == 'kstest'){
      return(ks.test(pway.vals, global,exact = F)$p.value)
    }
  }
  
  if (base_method=='Fstat'){
    fstat <- apply(dat.detect.w, 1, function(y, x) {
      anova(lm(y ~ x))[[4]][1]
    }, x = class.vector)
    fstat <- log(fstat, 2)
    t.pvals <- apply(incidence.matrix, 1, evalPway, global = fstat,sec_method=sec_method)
  }else if (base_method=='Tstat'){
    tstat <- apply(dat.detect.w, 1, function(y, x) {
      t.test(y ~ x)$statistic
    }, x = class.vector)
    tstat <- log(abs(tstat), 2)
    t.pvals <- apply(incidence.matrix, 1, evalPway, global = tstat,sec_method=sec_method)
  }else{
    stop('The method indicator base_method is invalid.')
  }
  t.pvals_adjust <- p.adjust(t.pvals, "BH")
  size <- apply(incidence.matrix, 1, sum)
  tab <- data.frame(MCBID = rownames(incidence.matrix),
                    annotation[sapply(X = rownames(incidence.matrix),FUN = function(x){strsplit(x, "_")[[1]][2]}),],
                    Pvalues = t.pvals,
                    AdjustedPvalues = t.pvals_adjust, 
                    NumberDetectedCpGs = size)
  #rownames(tab)<-sapply(X = rownames(tab),FUN = function(x){strsplit(x, "_")[[1]][2]})
  tab <- tab[order(t.pvals), ]
  return(tab)
}

buildIncidenceMatrix<-function (gene.ids, annotation) 
{
  CpGs_names<-annotation[,'CpGs']
  pway.genes <- strsplit(CpGs_names," ")
  convert.to.row <- function(x.genes, row.genes) {
    res <- integer(length(row.genes))
    res[row.genes %in% x.genes] <- 1
    return(res)
  }
  xmat <- t(sapply(lapply(pway.genes, convert.to.row, gene.ids), 
                   cbind))
  rownames(xmat) <- paste('MCB',annotation[,'MCB_no'],sep  = "_") 
  colnames(xmat) <- gene.ids
  return(xmat)
}

re_AnnotatedMMB<-function(MMB_list){
  Illumina_Infinium_Human_Methylation_450K <- get450kAnno()
  met_cg_allgene <- Illumina_Infinium_Human_Methylation_450K
  res<-rep(NA,7)
  names(res)<-c('chromosomes','start_site','end_site','CpGs','location','length','CpGs_num')
  res_all <-NULL
  for (MMB in MMB_list) {
    MMBannotated<-met_cg_allgene[MMB,]
    res['chromosomes'] <- unique(MMBannotated$chr)
    res['start_site']  <- min(MMBannotated$pos)
    res['end_site']  <- max(MMBannotated$pos)
    res['CpGs'] <-paste(MMBannotated$Name[order(MMBannotated$pos,decreasing = F)],collapse = " ")
    res['location'] <-paste(res['chromosomes'],":",res['start_site'],'-',res['chromosomes'],':',res['end_site'],sep = " ")
    res['length'] <- as.character(max(MMBannotated$pos)- min(MMBannotated$pos) )
    res['CpGs_num'] <- nrow(MMBannotated)
    res_all<-rbind(res_all,res)
  }
  rownames(res_all)<-1:nrow(res_all)
  res_all
}

bmeasures_otu<-
  function (x, y) 
  {
    dx <- dim(x)
    dy <- dim(y)
    if (dx[1] != dy[1]) {
      cat("Inputs have different number of rows \n")
    }
    else if (dx[2] != dy[2]) {
      cat("Inputs have different number of columns \n")
    }
    else {
      cat("")
    }
    input <- as.matrix(rbind(x, y))
    a <- 0
    b <- 0
    c <- 0
    d <- 0
    for (i in 1:dx[2]) {
      k <- input[1, i]
      l <- input[2, i]
      if ((k == 1) & (l == 1)) {
        a <- a + 1
      }
      else if ((k == 0) & (l == 1)) {
        b <- b + 1
      }
      else if ((k == 1) & (l == 0)) {
        c <- c + 1
      }
      else {
        d <- d + 1
      }
    }
    otu_out <- c(a = a, b = b, c = c, d = d)
    remove(a, b, c, d)
    return(otu_out)
  }

bmeasures <-
  function (x, y, method = 'eq_01') 
  {
    x<-matrix(x,1)
    y<-matrix(y,1)
    otuTable <- bmeasures_otu(x, y)
    a <- otuTable[1]
    b <- otuTable[2]
    c <- otuTable[3]
    d <- otuTable[4]
    n <- a + b + c + d
    switch(method, eq_01 = {
      coef <- a/(a + b + c)
    }, eq_02 = {
      coef <- a/(2 * a + b + c)
    }, eq_03 = {
      coef <- (2 * a)/(2 * a + b + c)
    }, eq_04 = {
      coef <- (3 * a)/(3 * a + b + c)
    }, eq_05 = {
      coef <- (2 * a)/((a + b) + (a + c))
    }, eq_06 = {
      coef <- a/(a + (2 * b) + (2 * c))
    }, eq_07 = {
      coef <- (a + d)/n
    }, eq_08 = {
      coef <- (2 * (a + d))/((2 * a) + b + c + (2 * d))
    }, eq_09 = {
      coef <- (a + d)/(a + (2 * (b + c)) + d)
    }, eq_10 = {
      coef <- (a + (0.5 * d))/n
    }, eq_11 = {
      coef <- (a + d)/(a + (0.5 * (b + c)) + d)
    }, eq_12 = {
      coef <- a
    }, eq_13 = {
      coef <- a + d
    }, eq_14 = {
      coef <- a/n
    }, eq_15 = {
      coef <- b + c
    }, eq_16 = {
      coef <- sqrt(b + c)
    }, eq_17 = {
      coef <- sqrt((b + c)^2)
    }, eq_18 = {
      coef <- sqrt((b + c)^2)
    }, eq_19 = {
      coef <- b + c
    }, eq_20 = {
      coef <- (b + c)/n
    }, eq_21 = {
      coef <- b + c
    }, eq_22 = {
      coef <- b + c
    }, eq_23 = {
      coef <- (b + c)/(4 * n)
    }, eq_24 = {
      coef <- ((b + c)^2)/(n^2)
    }, eq_25 = {
      coef <- ((n * (b + c)) - ((b - c)^2))/(n^2)
    }, eq_26 = {
      coef <- (4 * b * c)/(n^2)
    }, eq_27 = {
      coef <- (b + c)/(2 * a + b + c)
    }, eq_28 = {
      coef <- (b + c)/(2 * a + b + c)
    }, eq_29 = {
      coef <- 2 * (sqrt(1 - (a/sqrt((a + b) * (a + c)))))
    }, eq_30 = {
      coef <- (sqrt(2 * (1 - (a/(sqrt((a + b) * (a + c)))))))
    }, eq_31 = {
      coef <- a/(sqrt((a + b) * (a + c)))
    }, eq_32 = {
      coef <- log(a) - log(n) - log((a + b)/n) - log((a + c)/n)
    }, eq_33 = {
      coef <- a/(sqrt((a + b) * (a + c)))
    }, eq_34 = {
      coef <- (n * a)/((a + b) * (a + c))
    }, eq_35 = {
      coef <- (n * ((a - 0.5)^2))/((a + b) * (a + c))
    }, eq_36 = {
      coef <- (a^2)/((a + b) * (a + c))
    }, eq_37 = {
      coef <- a/(0.5 * (a * b + a * c) + (b * c))
    }, eq_38 = {
      coef <- a/(((a + b) * (a + c))^0.5)
    }, eq_39 = {
      coef <- ((a^2) - (b * c))/((a + b) * (a + c))
    }, eq_40 = {
      coef <- ((n * a) - (a + b) * (a + c))/((n * a) + (a + 
                                                          b) * (a + c))
    }, eq_41 = {
      coef <- ((a/2) * (2 * a + b + c))/((a + b) * (a + c))
    }, eq_42 = {
      coef <- (a/2) * ((1/(a + b)) + (1/(a + c)))
    }, eq_43 = {
      coef <- (a/(a + b)) + (a/(a + c))
    }, eq_44 = {
      coef <- ((a * d) - (b * c))/(sqrt(n * (a + b) * (a + 
                                                         c)))
    }, eq_45 = {
      coef <- a/(min((a + b), (a + c)))
    }, eq_46 = {
      coef <- a/(max((a + b), (a + c)))
    }, eq_47 = {
      coef <- (a/sqrt((a + b) * (a + c))) - (max((a + b), (a + 
                                                             c))/2)
    }, eq_48 = {
      coef <- ((n * a) - ((a + b) * (a + c)))/((n * min(a + 
                                                          b, a + c)) - (a + b) * (a + c))
    }, eq_49 = {
      coef <- 0.25 * ((a/(a + b)) + (a/(a + c)) + (d/(b + d)) + 
                        (d/(c + d)))
    }, eq_50 = {
      coef <- (a + d)/(sqrt((a + b) * (a + c) * (b + d) * (c + 
                                                             d)))
    }, eq_51 = {
      x2 <- (n * (((a * d) - (b * c))^2))/((a + b) * (a + c) * 
                                             (c + d) * (b + d))
      coef <- x2
    }, eq_52 = {
      x2 <- (n * (((a * d) - (b * c))^2))/((a + b) * (a + c) * 
                                             (c + d) * (b + d))
      coef <- sqrt(x2/(n + x2))
    }, eq_53 = {
      p <- ((a * d) - (b * c))/(sqrt((a + b) * (a + c) * (b + 
                                                            d) * (c + d)))
      coef <- sqrt(p/(n + p))
    }, eq_54 = {
      p <- ((a * d) - (b * c))/(sqrt((a + b) * (a + c) * (b + 
                                                            d) * (c + d)))
      coef <- p
    }, eq_55 = {
      coef <- cos((pi * sqrt(b * c))/(sqrt(a * d) + sqrt(b * 
                                                           c)))
    }, eq_56 = {
      coef <- (a + d)/(b + c)
    }, eq_57 = {
      coef <- (a * d)/(((a + b) * (a + c) * (b + d) * (c + 
                                                         d))^0.5)
    }, eq_58 = {
      coef <- (sqrt(2) * (a * d - b * c))/(sqrt(((a * d - b * 
                                                    c)^2) - (a + b) * (a + c) * (b + d) * (c + d)))
    }, eq_59 = {
      coef <- log10((n * (((abs(a * d - b * c)) - n/2)^2))/((a + 
                                                               b) * (a + c) * (b + d) * (c + d)))
    }, eq_60 = {
      coef <- (a * d)/(sqrt((a + b) * (a + c) * (b + d) * (c + 
                                                             d)))
    }, eq_61 = {
      coef <- (a * d - b * c)/(a * d + b * c)
    }, eq_62 = {
      coef <- ((2 * b * c)/(a * d + b * c))
    }, eq_63 = {
      coef <- (sqrt(a * d) - sqrt(b * c))/(sqrt(a * d) + sqrt(b * 
                                                                c))
    }, eq_64 = {
      coef <- a/(b + c)
    }, eq_65 = {
      coef <- a/((a + b) + (a + c) - a)
    }, eq_66 = {
      coef <- (a * d - b * c)/(n^2)
    }, eq_67 = {
      coef <- ((a + d) - (b + c))/n
    }, eq_68 = {
      coef <- (4 * (a * d - b * c))/(((a + d)^2) + ((b + c)^2))
    }, eq_69 = {
      sig <- max(a, b) + max(c, d) + max(a, c) + max(b, d)
      sigt <- max(a + c, b + d) + max(a + b, c + d)
      coef <- (sig - sigt)/(2 * n - sigt)
    }, eq_70 = {
      sig <- max(a, b) + max(c, d) + max(a, c) + max(b, d)
      sigt <- max(a + c, b + d) + max(a + b, c + d)
      coef <- (sig - sigt)/(2 * n)
    }, eq_71 = {
      coef <- (sqrt(a * d) + a)/(sqrt(a * d) + a + b + c)
    }, eq_72 = {
      coef <- (sqrt(a * d) + a - (b + c))/(sqrt(a * d) + a + 
                                             b + c)
    }, eq_73 = {
      coef <- (a * b + b * c)/((a * b) + (2 * b * c) + (c * 
                                                          d))
    }, eq_74 = {
      coef <- ((n^2) * (n * a - (a + b) * (a + c)))/((a + b) * 
                                                       (a + c) * (b + d) * (c + d))
    }, eq_75 = {
      coef <- (a * (c + d))/(c * (a + b))
    }, eq_76 = {
      coef <- abs((a * (c + d))/(c * (a + b)))
    }, eq_77 = {
      coef <- log(1 + a)/log(1 + n)
    }, eq_78 = {
      coef <- log(1 + a)/log(1 + a + b + c)
    }, eq_79 = {
      coef <- (log(1 + a * d) - log(1 + b * c))/log(1 + (n^2)/4)
    }, {
      cat("No desired equation. Please check it again.")
    })
    if (is.na(coef)) coef = 1
    result <- c(a, b, c, d, coef)
    result <- t(as.matrix(result))
    colnames(result) <- c("a", "b", "c", "d", 
                          "coef")
    return(coef)
  }

vector_to_matrix<-function(x){
  list_data<-x[grep('X',names(x))]
  matrix_data <-unlist((lapply(list_data,function(x){strsplit(x," ")[[1]]})))
  mat<-as.numeric_matrix(matrix(matrix_data,nrow = length(matrix_data)/14,ncol = 14))
  rownames(mat)<-strsplit(as.character(x['CpGs']),' ')[[1]]
  mat
}

fisher_combine_p<-function (p,...) 
{
  k <- length(p)
  adjust <- "none"
  statistic <- -2 * sum(log(p))
  pval <- pchisq(statistic, df = 2 * k, lower.tail = FALSE)
  return(pval)
}


# end load

make_eset<-function(matrix,pData){
  requireNamespace("affycoretools")

  colnames<-rownames(pData)

  commdata<-intersect(colnames,colnames(matrix))

  pData<-pData[rownames(pData) %in% commdata,]

  matrix<-matrix[,colnames(matrix) %in% commdata]

  if (nrow(pData)!=ncol(matrix)){stop("matrix colnames do not match the pData rownames")}

  #pData<-pData[colnames(matrix),]
  matrix<-matrix[,rownames(pData)]

  metadata<-data.frame(labelDescription=colnames(pData),stringsAsFactors = FALSE)

  adf<-new("AnnotatedDataFrame",data=pData,varMetadata=metadata)

  eSet<-new("ExpressionSet",exprs=as.matrix(matrix),phenoData=adf)

  return(eSet)
}

#' @title fitting function using stacking ensemble model for Methylation Correlation Block
#'
#' @description predict is a generic function for predictions from the results of stacking ensemble model fitting functions.
#' The function invokes particular methods which is the ensemble model described in the reference.
#' @param ensemble_model ensemble model which built by ensemble_model() function
#' @param prediction_data A vector, matrix, list, or data frame containing the predictions (input).
#' @param multiple_results Boolean vector, True for including the single model results.
#' @references
#' Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
#' @export
#' @return Object of numeric class \code{double}
#' @examples 
#' library(survival)
#' #import datasets
#' data(demo_survival_data)
#' datamatrix<-create_demo()
#' data(demo_MCBinformation)
#' #select MCB with at least 3 CpGs.
#' demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]
#' trainingset<-colnames(datamatrix) %in% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
#' testingset<-!trainingset
#' #select one MCB
#' select_single_one=1
#' em<-ensemble_model(t(demo_MCBinformation[select_single_one,]),
#'     training_set=datamatrix[,trainingset],
#'     Surv_training=demo_survival_data[trainingset])
#'
#' em_prediction_results<-ensemble_prediction(ensemble_model = em,
#' prediction_data = datamatrix[,testingset])
#'
ensemble_prediction <- function(ensemble_model,prediction_data, multiple_results = FALSE) {
  if (multiple_results) {
    return(ensemble_prediction.m(ensemble_model,prediction_data))
  }
  prediction_data<-prediction_data[ensemble_model$cox$cox_model$CpGs,]
  if (nrow(prediction_data)!=length(rownames(prediction_data))) {
    stop("ERROR: The predition data and the model have wrong dimensions!")
  }
  svm<-stats::predict(ensemble_model$svm$svm_model, data.frame(t(prediction_data)))$predicted
  cox<-stats::predict(ensemble_model$cox$cox_model, data.frame(t(prediction_data)))
  enet<-stats::predict(ensemble_model$enet$enet_model,t(prediction_data),s=ensemble_model$enet$`corrected_lambda(min)`)
  mboost<-stats::predict(ensemble_model$mboost$mboost_model, t(prediction_data))[,1]
  data<-rbind(cox,
              svm,
              t(enet),
              mboost
  )
  rownames(data)<-c('cox','svm','enet','mboost')
  data<-t(data)
  data_f<-as.data.frame(data)
  if (class(ensemble_model$stacking)[1] == "cv.glmnet"){
    if (is.null(ensemble_model$stacking$ensemble_type)){
      return(stats::predict(ensemble_model$stacking, as.matrix(data_f)))
    }else{
      f1<-apply(data_f,1,function(x)e1071::kurtosis(x))
      f2<-apply(data_f,1,function(x)(-log(sd(x),2)))
      data_f<-data.frame(f1cox=data_f$cox*f1,f2cox=data_f$cox*f2,
                         f1svm=data_f$svm*f1,f2svm=data_f$svm*f2,
                         f1enet=data_f$enet*f1,f2enet=data_f$enet*f2,
                         f1mboost=data_f$mboost*f1,f2mboost=data_f$mboost*f2)
      return(stats::predict(ensemble_model$stacking, as.matrix(data_f)))
    }
  }
  else if (class(ensemble_model$stacking)[1] == "cph")
    return(stats::predict(ensemble_model$stacking, as.matrix(data_f)))
  else
    stop("ERROR: The ensemble predition model is invaild !")
    
}

## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------
#if(!requireNamespace("BiocManager", quietly = TRUE))
#    install.packages("BiocManager")
#BiocManager::install("EnMCB")

## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------

suppressPackageStartupMessages(library(EnMCB))

methylation_dataset<-create_demo()

res<-IdentifyMCB(methylation_dataset)


## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------

MCB<-res$MCBinformation


## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------

MCB<-MCB[MCB[,"CpGs_num"]>=5,]


## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------
#simulation for the group data
groups = c(rep("control",200),rep("dis",255))

DiffMCB_resutls<-DiffMCB(methylation_dataset,
                         groups,
                         MCB)$tab


## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------
# sample the dataset into training set and testing set
trainingset<-colnames(methylation_dataset) %in% sample(colnames(methylation_dataset),0.6*length(colnames(methylation_dataset)))

testingset<-!trainingset

#build the models
library(survival)
data(demo_survival_data)

models<-metricMCB(MCB,
                    training_set = methylation_dataset[,trainingset],
                    Surv = demo_survival_data[trainingset],
                    Method = "cox",ci = TRUE)

#select the best
onemodel<-models$best_cox_model$cox_model


## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------
newcgdata<-data.frame(t(methylation_dataset[,testingset]))
           
prediction_results<-predict(onemodel, newcgdata)

## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------
# You can choose one of MCBs:
select_single_one=1

em<-ensemble_model(t(MCB[select_single_one,]),
                    training_set=methylation_dataset[,trainingset],
                    Surv_training=demo_survival_data[trainingset])
                    

## ----echo=TRUE,warning=FALSE,results='hide'-----------------------------------
em_prediction_results<-ensemble_prediction(ensemble_model = em,
                    prediction_data = methylation_dataset[,testingset])

## ----echo=TRUE----------------------------------------------------------------
sessionInfo()

#' @title Calculation of model AUC for Methylation Correlation Blocks using cross validation
#'
#' @description To enable quantitative analysis of the methylation patterns
#' within individual Methylation Correlation Blocks across many samples, a single metric to
#' define the methylated pattern of multiple CpG sites within each block.
#' Compound scores which calculated all CpGs within individual Methylation Correlation Blocks by SVM model
#' were used as the compound methylation values of Methylation Correlation Blocks.
#' @usage metricMCB.cv(MCBset,data_set,Surv,nfold,
#' Method,predict_time,alpha,n_mstop,n_nu,theta,silent)
#' @export
#' @param MCBset Methylation Correlation Block information returned by the IndentifyMCB function.
#' @param data_set methylation matrix used for training the model in the analysis.
#' @param Surv Survival function contain the survival information for training.
#' @param nfold fold used in the cross validation precedure.
#' @param Method model used to calculate the compound values for multiple Methylation correlation blocks. Options include "svm", "cox", "mboost", and "enet". The default option is SVM method.
#' @param predict_time time point of the ROC curve used in the AUC calculations, default is 3 years.
#' @param alpha The elasticnet mixing parameter, with 0 <= alpha <= 1. alpha=1 is the lasso penalty, and alpha=0 the ridge penalty. It works only when "enet" Method is selected.
#' @param n_mstop an integer giving the number of initial boosting iterations. If mstop = 0, the offset model is returned. It works only when "mboost" Method is selected.
#' @param n_nu a double (between 0 and 1) defining the step size or shrinkage parameter in mboost model. It works only when "mboost" Method is selected.
#' @param theta penalty used in the penalized coxph model, which is theta/2 time sum of squared coefficients. default is 1. It works only when "cox" Method is selected.
#' @param silent Ture indicates that processing information and progress bar will be shown.
#' @author Xin Yu
#' @keywords Methylation Correlation
#' @examples
#' #import datasets
#' data(demo_survival_data)
#' datamatrix<-create_demo()
#' data(demo_MCBinformation)
#' #select MCB with at least 3 CpGs.
#' demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]
#'
#' trainingset<-colnames(datamatrix) %in% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
#' testingset<-!trainingset
#' #create the results using Cox regression. 
#' mcb_cox_res<-metricMCB.cv(MCBset = demo_MCBinformation,
#'                data_set = datamatrix,
#'                Surv = demo_survival_data,
#'                Method = "cox")
#'
#' @return Object of class \code{list} with elements (XXX will be replaced with the model name you choose):
#'  \tabular{ll}{
#'    \code{MCB_matrix} \tab Prediction results of model. \cr
#'    \code{auc_results} \tab AUC results for each model. \cr
#'  }
#' @references
#' Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
#' @importFrom mboost glmboost predict.glmboost CoxPH boost_control
#' 

metricMCB.cv<-function(
  MCBset,
  data_set,
  Surv,
  nfold=10,
  Method=c("svm","cox","enet","mboost")[1],
  predict_time = 3,
  alpha = 0.5,
  n_mstop = 500,
  n_nu = 0.1,
  theta = 1,
  silent=FALSE
){
  requireNamespace("stats")
  if (!silent) {
    cat("Start anaylsis, this may take a while...\n")
    show_bar=nrow(MCBset)>1
  }else{
    show_bar=FALSE
  }
  if (show_bar) {
    bar<-utils::txtProgressBar(min = 1,max = nrow(MCBset),char = "#",style = 3)
  }
  if (is.null(Surv)) {
    stop(paste("You must have a survival function to train the data."))
  }
  if (is.integer0(grep("MCB_no|CpGs",colnames(MCBset)))){
    stop(paste("Methylation Correlation Block information in your result must have columns of MCB_no and CpGs. Please check your results."))
  }
  na_or_zero_data<-(is.na(Surv)|Surv[,1]==0)
  if (sum(na_or_zero_data)>0){
    data_set<-data_set[,!na_or_zero_data]
    Surv<-Surv[!na_or_zero_data]
    warning("survival data contains NAs or zero survival times, NAs or data with zero survival times are remove automaticly.")
  }
  # constuction of MCB Method matrix for SVM
  MCB_matrix<-matrix(0,nrow = nrow(MCBset),ncol = ncol(data_set))
  colnames(MCB_matrix)<-colnames(data_set)
  rownames(MCB_matrix)<-as.numeric(MCBset[,'MCB_no'])
  if (!Method %in% c("svm","cox","enet","mboost")){
    stop(paste("Method:",Method,"is not supported, see hlep files for the details.",collapse = " "))
  }
  sp<-sample(1:ncol(data_set),replace = F)
  order_sp<-order(sp)
  data_set<-data_set[,sp]
  folds <- cut(seq(1,ncol(data_set)),breaks=nfold,labels=FALSE)
  #if it has a independent test set create the test_set res set
  FunctionResults<-NULL
  best_auc<-0
  best_model<-NULL
  mcb_model_res<-NULL
  for (mcb in seq_len(nrow(MCBset))) {
    #if (nrow(MCBset)>1){}
    if (show_bar&!silent) {
      utils::setTxtProgressBar(bar, mcb)
    }
    write_MCB<-rep(NA,5)
    #save the mcb number
    write_MCB[1]<-as.numeric(MCBset[mcb,'MCB_no'])
    write_MCB[2]<-MCBset[mcb,'CpGs']
    # build temp variable for saving the results.
    # MCB number
    # aquire information for CpG sites in MCB
    CpGs<-strsplit(MCBset[mcb,'CpGs']," ")[[1]]
    #cat(CpGs)
    for (i in seq(unique(folds))) {
      rz<- which(folds==i,arr.ind=TRUE)
      data_used_for_training<-data.frame(t(data_set[CpGs,-rz]))
      data_used_for_testing <-data.frame(t(data_set[CpGs,rz]))
      # train a svm model
      times = Surv[-rz]
      if (Method=="svm") {
        model<-tryCatch(survivalsvm::survivalsvm(times ~ .,
                                                 data_used_for_training,
                                                 gamma.mu = 0.1,
                                                 type = "regression"),
                        error = function(e){warning(paste('SVR can not be built, error occurs:', e));return(NULL)})
      }else if(Method=="cox"){
        data_used_for_training = data.frame(allvars = as.ridgemat(data_used_for_training))
        if (length(CpGs)<5){
          model<-tryCatch(survival::coxph(times ~ allvars, 
                                data_used_for_training),
                error = function(e){return(NULL)})
        }else{
          model<-tryCatch(ridge_model(times, data_used_for_training, theta),
                          error = function(e){return(NULL)})
        }
        if (!is.null(model)){
          model$CpGs <- CpGs
          model <- as.mcb.coxph.penal(model)
        }
      }else if(Method=="enet"){
        model<-tryCatch( glmnet::cv.glmnet(data_used_for_training,
                                           times,
                                           #cox model in enet was used, note that here penalty were used.
                                           family="cox",
                                           alpha=alpha,
                                           # The elasticnet mixing parameter, with 0≤α≤ 1. The penalty is defined as
                                           # (1-alpha)/2||beta||_2^2+alpha||beta||_1
                                           # alpha=1 is the lasso penalty, and alpha=0 the ridge penalty.
                                           # type.measure = "AUC"
                                           type.measure= "deviance"
                                           # It uses AUC as the criterion for 10-fold cross-validation.
                                           #foldid = 10
        ),error = function(e){return(NULL)})
        correctional_value=1
        while ( sum(stats::coef(model, s = model$lambda.min-0.001*(correctional_value-1))>0)<1 &
                (model$lambda.min-0.001*(correctional_value-1))>0 ) {
          correctional_value=correctional_value*1.25
        }
        lambda_min_corrected<-model$lambda.min-0.001*(correctional_value-1)
      }else if (Method=="mboost"){
        model <- tryCatch(mboost::glmboost(y=times,x=data_used_for_training,family=mboost::CoxPH(),
                                                    control=mboost::boost_control(mstop=n_mstop,nu=n_nu)),error = NULL)
      }
      if (!is.null(model)) {
        #predictions
        if (Method=="svm") MCB_matrix[mcb,rz]<-stats::predict(model,data_used_for_testing)$predicted
        if (Method=="cox") MCB_matrix[mcb,rz]<-stats::predict(model,data_used_for_testing)
        if (Method=="enet"){
          #if you use lambda.1se instead, the penalty of enet would be larger, leading that most of covariates were removed form the final model.
          MCB_matrix[mcb,rz]<-stats::predict(model,data_used_for_testing,s=lambda_min_corrected)
        }
        if (Method=="mboost")MCB_matrix[mcb,rz] <- stats::predict(model, data_used_for_testing)[,1]
      }else{
        MCB_matrix[mcb,rz]<-NA
      }
    }
    MCB_matrix[mcb,]<-MCB_matrix[mcb,order_sp]
    if (sum(is.na(MCB_matrix[mcb,])) == 0){
      if (Method == "cox"){
        marker_hr <- predict(survival::coxph(Surv~MCB_matrix[mcb,]))
        AUC_value<-survivalROC::survivalROC(Stime = Surv[,1],
                                            status = Surv[,2],
                                            marker = marker_hr,
                                            predict.time = predict_time,
                                            method = "NNE",
                                            span =0.25*length(Surv)^(-0.20))$AUC
        reg<-survival::survreg(Surv ~ marker_hr)
        cindex<-survival::concordance(reg)
      }else{
      AUC_value<-survivalROC::survivalROC(Stime = Surv[,1],
                                          status = Surv[,2],
                                          marker = MCB_matrix[mcb,],
                                          predict.time = predict_time,
                                          method = "NNE",
                                          span =0.25*length(Surv)^(-0.20))$AUC
      reg<-survival::survreg(Surv ~ MCB_matrix[mcb,])
      cindex<-survival::concordance(reg)
      }
      write_MCB[3]<-AUC_value
      write_MCB[4]<-cindex$concordance
      write_MCB[5]<-cindex$cvar
    }else{
      write_MCB[3:5]<-NA
    }
    mcb_model_res<-rbind(mcb_model_res,write_MCB)
  }
  colnames(mcb_model_res)<-c("MCB_no","CpGs","auc","C-index","C-index_SE")
  FunctionResults$MCB_matrix<-MCB_matrix
  FunctionResults$auc_results<-mcb_model_res
  return(FunctionResults)
}

#' @title multivariate survival analysis using coxph
#' @author Xin Yu
#'
#' @param dataframe Clinic data and covariates ready to be tested. Note that Rows are samples and columns are variables.
#' @param y_surv Survival function contain survival data, usually are obtained form Surv() function in survival package.
#' @param digits Integer indicating the number of decimal places.
#' @param asnumeric indicator that the data will be (True) / not (False) transformed into numeric. Default is true.
#' @export
#'
#' @examples
#' data(demo_survival_data)
#' data('demo_data',package = "EnMCB")
#' demo_set<-demo_data$realdata
#' res<-multi_coxph(t(demo_set),demo_survival_data)
#' @return Object of class \code{matrix} with results.
#' 
multi_coxph <- function(dataframe,y_surv,digits=4,asnumeric=TRUE) {
  dataframe <- as.data.frame(dataframe)
  if (asnumeric) {
    for (i in seq(ncol(dataframe))) {
      if (!is.numeric(dataframe[1,i]))  dataframe[,i] <-as.numeric(as.factor(dataframe[,i] ))
    }
  }
  #print(head(dataframe))
  covariates <- paste(colnames(dataframe),collapse = ' + ')
  formula_str<- paste('y_surv',covariates,sep = ' ~ ')
  #cat(formula_str)
  multi_models<-survival::coxph(as.formula(formula_str),data = dataframe)
  multi_models
}

#' @import stats
#' @import survivalROC
#' @import glmnet
#' @import survivalsvm
#' @import ggplot2
#' @import survival
#' @import utils
#' @import BiocFileCache
#' @importFrom methods as
#' @importFrom mboost CoxPH
#' @importFrom mboost boost_control
#' @importFrom mboost glmboost
#' @importFrom mboost predict.glmboost



utils::globalVariables(c('FPF','TPF','aes','chr_id','coef','coord_cartesian','doParallel','data','anno_matrix',
                         'element_blank','element_line','foreach','geom_abline','geom_roc','ggplot','rnorm',
                         'ggplot2','ggsave','groups','median','new','parallel','pheatmap','plot','demo_data',
                         'prediction','prognosticROC','qnorm','style_roc','survivalROC','theme','boot','boot.ci',
                         'AdjustedPvalues','theme_grey','theme_light'))

#' @title  Compare multiple methylation correlated blocks lists
#'
#' @description This function is used to find the Methylation correlated blocks that differentially expressed between groups. 
#' This function calculates attractors of all the MCBs among the groups and find the attractor MCBs. \cr
#'
#' @details Currently, only illumina 450k platform is supported, the methylation profile need to convert into matrix format.
#'
#' @param MCBs Methylation correlated blocks list.
#' @param method method used for calculation of differential expression, \cr
#' should be one of "attractors","t-test". Defualt is "attractors".
#' @param p_value p value threshold for the test.
#' @param min_CpGs threshold for minimum CpGs must included in the individual MCBs.
#' @param platform This parameter indicates the platform used to produce the methlyation profile.
#'
#' @author Xin Yu
#' @return
#' Object of class \code{list} with elements:
#'  \tabular{ll}{
#'    \code{MCBsites} \tab Character set contains all CpG sites in MCBs. \cr
#'    \code{MCBinformation} \tab Matrix contains the information of results. \cr
#'  }
#' @examples
#' data('demo_data',package = "EnMCB")
#' 
#'
#' @export
#'
#' @references
#' Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415
#'
#'
CompareMCB<-function(
  MCBs,
  method=c("attractors")[1],
  p_value = 0.05,
  min_CpGs = 5,
  platform = "Illumina Methylation 450K"
){
    union_cpgs<-list()
    i = 1
    for (MCB in MCBs) {
      nrow_MCB<-nrow(MCB)
      if (nrow_MCB==0){
        stop("compare MCB: the MCB info matrix doesn't include any data, please check the data.")
      }
      for (j in seq(nrow_MCB)) {
        CpG_list<-strsplit(paste(MCB[j, "CpGs"], collapse = " "), " ")[[1]]
        if (length(CpG_list)>=min_CpGs){
          union_cpgs[[i]]<-CpG_list
          i = i + 1
        }
      }
    }
    i = rep(1:length(union_cpgs), lengths(union_cpgs)) 
    j = factor(unlist(union_cpgs))
    tab = Matrix::sparseMatrix(i = i, j = as.integer(j), x = TRUE, dimnames = list(NULL, levels(j)))
    connects = Matrix::tcrossprod(tab, boolArith = TRUE)
    # 'graph_from_adjacency_matrix' seems to not work with the "connects" object directly. 
    # An alternative to coercing "connects" here would be to build it as 'tcrossprod(tab) > 0'
    group = igraph::clusters(igraph::graph_from_adjacency_matrix(as(connects, "lsCMatrix")))$membership
    #group
    #[1] 1 2 2 2 3 3
    CpGs_MMB<-tapply(union_cpgs, group, function(x) sort(unique(unlist(x))))
    ref_MMB<-re_AnnotatedMMB(CpGs_MMB)
    patterns<-matrix(NA,nrow = nrow(ref_MMB),ncol = length(MCBs))
    bscore<-rep(NA,nrow(ref_MMB))
    bar<-utils::txtProgressBar(min = 1,max = nrow(ref_MMB),char = "#",style = 3)
    for (i in seq(nrow(ref_MMB))) {
      utils::setTxtProgressBar(bar, i)
      CpG_list<-strsplit(paste(ref_MMB[i,'CpGs'], collapse = " "), " ")[[1]]
      pattern_codes<-NULL
      for (j in seq(length(MCBs))) {
        pattern_code<-NULL
        for (CpG in CpG_list) {
          
          if (is.integer0(grep(CpG,MCBs[[j]][,'CpGs']))){
            pattern_code<-c(pattern_code,0)
          }else{
            pattern_code<-c(pattern_code,1)
          }
        }
        patterns[i,j]<-paste(pattern_code,collapse = ' ')
        pattern_codes[[j]]<-pattern_code
      }
      bscore[i] <- mean(outer(pattern_codes,pattern_codes,Vectorize(bmeasures))
                        [upper.tri( matrix(NA,
                                           nrow = length(pattern_codes),
                                           ncol = length(pattern_codes))  ) ])
    }
    MCB_no<-seq(nrow(ref_MMB))
    data.frame(MCB_no,ref_MMB,patterns,bscore)
}

#' @title Calculation of the metric matrix for Methylation Correlation Block
#'
#' @description To enable quantitative analysis of the methylation patterns \cr
#' within individual Methylation Correlation Blocks across many samples, a single metric to \cr
#' define the methylated pattern of multiple CpG sites within each block.\cr
#' Compound scores which calculated all CpGs within individual Methylation Correlation Blocks by linear, SVM or elastic-net model\cr
#' Predict values were used as the compound methylation values of Methylation Correlation Blocks.
#' @usage metricMCB(MCBset,training_set,Surv,testing_set,
#' Surv.new,Method,predict_time,ci,silent,alpha,n_mstop,n_nu,theta)
#' @export
#' @param training_set methylation matrix used for training the model in the analysis.
#' @param testing_set methylation matrix used in the analysis. This can be missing then training set itself will be used as testing set.
#' @param MCBset Methylation Correlation Block information returned by the IndentifyMCB function.
#' @param Surv Survival function contain the survival information for training.
#' @param Surv.new Survival function contain the survival information for testing.
#' @param Method model used to calculate the compound values for multiple Methylation correlation blocks. \cr
#' Options include "svm" "cox" "mboost" and "enet". The default option is SVM method.
#' @param predict_time time point of the ROC curve used in the AUC calculations, default is 5 years.
#' @param ci if True, the confidence intervals for AUC under area under the receiver operating characteristic curve will be calculated. This will be time consuming. default is False.
#' @param silent True indicates that processing information and progress bar will be shown.
#' @param alpha The elasticnet mixing parameter, with 0 <= alpha <= 1. alpha=1 is the lasso penalty, and alpha=0 the ridge penalty. \cr
#' It works only when "enet" Method is selected.
#' @param n_mstop an integer giving the number of initial boosting iterations. If mstop = 0, the offset model is returned. \cr
#' It works only when "mboost" Method is selected.
#' @param n_nu a double (between 0 and 1) defining the step size or shrinkage parameter in mboost model. \cr
#' It works only when "mboost" Method is selected.
#' @param theta penalty used in the penalized coxph model, which is theta/2 time sum of squared coefficients. default is 1. \cr
#' It works only when "cox" Method is selected.
#' @author Xin Yu
#' @keywords Methylation Correlation
#' @examples
#' #import datasets
#' data(demo_survival_data)
#' datamatrix<-create_demo()
#' data(demo_MCBinformation)
#' #select MCB with at least 3 CpGs.
#' demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]
#'
#' trainingset<-colnames(datamatrix) %in% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
#' testingset<-!trainingset
#' #create the results using Cox regression. 
#' mcb_cox_res<-metricMCB(MCBset = demo_MCBinformation,
#'                training_set = datamatrix[,trainingset],
#'                Surv = demo_survival_data[trainingset],
#'                testing_set = datamatrix[,testingset],
#'                Surv.new = demo_survival_data[testingset],
#'                Method = "cox"
#'                )
#'
#' @return Object of class \code{list} with elements (XXX will be replaced with the model name you choose):
#'  \tabular{ll}{
#'    \code{MCB_XXX_matrix_training} \tab Prediction results of model for training set. \cr
#'    \code{MCB_XXX_matrix_test_set} \tab Prediction results of model for test set. \cr
#'    \code{XXX_auc_results} \tab AUC results for each model. \cr
#'    \code{best_XXX_model} \tab Model object for the model with best AUC. \cr
#'    \code{maximum_auc} \tab Maximum AUC for the whole generated models. \cr
#'  }
#' @references
#' Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
#' @importFrom mboost glmboost predict.glmboost CoxPH boost_control
metricMCB<-function(
  MCBset,
  training_set,
  Surv,
  testing_set=NULL,
  Surv.new=NULL,
  Method=c("svm","cox","enet","mboost")[1],
  predict_time = 5,
  ci=FALSE,
  silent=FALSE,
  alpha = 0.5,
  n_mstop = 500,
  n_nu = 0.1,
  theta = 1
  ){
  requireNamespace("stats")
  # load private functions
  bs_ci <- function(data, indices, predict.time = predict_time) { 
    d <- data[indices,] # allows boot to select sample 
    surv.res = survivalROC(Stime = d$survival, 
                           status = d$survival_status, 
                           marker = d$marker, 
                           predict.time = predict.time, method = "NNE",
                           span = 0.25*NROW(d)^(-0.20))
    return(surv.res$AUC)
  }
  # end load
  
  if (!silent) {
    cat("Start anaylsis, this may take a while...\n")
    show_bar=nrow(MCBset)>1
  }else{
    show_bar=FALSE
  }
  if (show_bar) {
    bar<-utils::txtProgressBar(min = 1,max = nrow(MCBset),char = "#",style = 3)
  }
  if (is.null(Surv)) {
    stop(paste("You must have a survival function to train the data."))
  }
  if (is.integer0(grep("MCB_no|CpGs",colnames(MCBset)))){
    stop(paste("Methylation Correlation Block information in your result must have columns of MCB_no and CpGs. Please check your results."))
  }
  #private functions
  create_doc<-function(ci){
    if (ci){
      write_MCB<-rep(NA,5)
      names(write_MCB)<-c('MCB_no','AUC_train','95_CI_train','AUC_test','95_CI_test')
      return(write_MCB)
    }else{
      write_MCB<-rep(NA,3)
      names(write_MCB)<-c('MCB_no','AUC_train','AUC_test')
      return(write_MCB)
    }
  }
  if (!Method %in% c("svm","cox","enet","mboost")){
    stop(paste("Method:",Method,"is not supported, see hlep files for the details.",collapse = " "))
  }else if (Method=="svm") {
    # constuction of MCB Method matrix for SVM
    MCB_svm_matrix_training<-matrix(0,nrow = nrow(MCBset),ncol = ncol(training_set))
    colnames(MCB_svm_matrix_training)<-colnames(training_set)
    rownames(MCB_svm_matrix_training)<-as.numeric(MCBset[,'MCB_no'])
    #if it has a independent test set create the test_set res set
    if (!is.null(testing_set)) {
      MCB_svm_matrix_test_set<-matrix(0,nrow = nrow(MCBset),ncol = ncol(testing_set))
      colnames(MCB_svm_matrix_test_set)<-colnames(testing_set)
      rownames(MCB_svm_matrix_test_set)<-as.numeric(MCBset[,'MCB_no'])
    }else{
      MCB_svm_matrix_test_set<-NULL
    }
    FunctionResults<-NULL
    rz=!(is.na(Surv)|Surv[,1]==0)
    times=Surv[rz]
    best_auc<-0
    best_model<-NULL
    mcb_SVM_res<-NULL
    for (mcb in seq_len(nrow(MCBset))) {
      #if (nrow(MCBset)>1){}
      if (show_bar&!silent) {
        utils::setTxtProgressBar(bar, mcb)
      }
      write_MCB<-create_doc(ci)
      #save the mcb number
      write_MCB['MCB_no']<-as.numeric(MCBset[mcb,'MCB_no'])
      # build temp variable for saving the results.
      # MCB number
      # aquire information for CpG sites in MCB
      CpGs<-strsplit(MCBset[mcb,'CpGs']," ")[[1]]
      data_used_for_training<-data.frame(t(training_set[CpGs,rz]))
      # train a svm model
      svm_model <- tryCatch(survivalsvm::survivalsvm(times ~ ., data_used_for_training, gamma.mu = 0.1,type = "regression"),error = NULL)
      #predictions
      if (!is.null(svm_model)) {
        MCB_svm_matrix_training[mcb,]<-stats::predict(svm_model, data.frame(t(training_set[CpGs,])))$predicted
        rank_svm <-MCB_svm_matrix_training[mcb,rz]
        hr_model<-survival::coxph(times~rank_svm)
        hr_svm_train<-predict(hr_model)
        auc_and_ci = calculate_auc_ci(survival = times,marker = hr_svm_train,predict_time,ci)
        write_MCB['AUC_train']<-auc_and_ci$AUC
        if (ci) write_MCB['95_CI_train']<-auc_and_ci$CI95
        #if it has a independent test set
        if (!is.null(testing_set)){
          MCB_svm_matrix_test_set[mcb,]<-stats::predict(svm_model, data.frame(t(testing_set[CpGs,])))$predicted
          hr_svm_test<-predict(hr_model,data.frame(rank_svm = MCB_svm_matrix_test_set[mcb,]))
          auc_and_ci = calculate_auc_ci(Surv.new,marker = hr_svm_test,predict_time,ci)
          write_MCB['AUC_test']<-auc_and_ci$AUC
          if (ci) write_MCB['95_CI_test']<-auc_and_ci$CI95
          if ((write_MCB['AUC_train']+write_MCB['AUC_test'])>best_auc){
            best_auc<-write_MCB['AUC_train']+write_MCB['AUC_test']
            best_model<-list(mcb,svm_model,hr_model)
          }
          #if it does not have a independent test set
        }else{
          write_MCB<-write_MCB[1:3]
          if (write_MCB['AUC_train']>best_auc){
            best_auc<-write_MCB['AUC_train']
            best_model<-list(mcb,svm_model,hr_model)
          }
        }
      }else{
        stop("This svmr model can not be built.")
      }
      mcb_SVM_res<-rbind(mcb_SVM_res,write_MCB)
    }
    colnames(mcb_SVM_res)<-c("MCB_no","training_set_auc","test_set_auc")
    names(best_model)<-c("MCB_no","svm_model","hr_model")
    FunctionResults$MCB_svm_matrix_training<-MCB_svm_matrix_training
    FunctionResults$MCB_svm_matrix_test_set<-MCB_svm_matrix_test_set
    FunctionResults$svm_auc_results<-mcb_SVM_res
    FunctionResults$maximum_auc<-best_auc
    FunctionResults$best_svm_model<-best_model
  }else if(Method=="cox"){
    # constuction of MCB Method matrix for cox
    MCB_cox_matrix_training<-matrix(0,nrow = nrow(MCBset),ncol = ncol(training_set))
    colnames(MCB_cox_matrix_training)<-colnames(training_set)
    rownames(MCB_cox_matrix_training)<-as.numeric(MCBset[,'MCB_no'])
    #if it has a independent test set create the test_set res set
    if (!is.null(testing_set)) {
      MCB_cox_matrix_test_set<-matrix(0,nrow = nrow(MCBset),ncol = ncol(testing_set))
      colnames(MCB_cox_matrix_test_set)<-colnames(testing_set)
      rownames(MCB_cox_matrix_test_set)<-as.numeric(MCBset[,'MCB_no'])
    }else{
      MCB_cox_matrix_test_set<-NULL
    }
    FunctionResults<-NULL
    rz=!(is.na(Surv)|Surv[,1]==0)
    times=Surv[rz]
    best_auc<-0
    best_model<-NULL
    mcb_cox_res<-NULL
    for (mcb in seq_len(nrow(MCBset))) {
      if (show_bar&!silent){utils::setTxtProgressBar(bar, mcb)}
      write_MCB<-create_doc(ci)
      #save the mcb number
      write_MCB['MCB_no']<-as.numeric(MCBset[mcb,'MCB_no'])
      # build temp variable for saving the results.
      # MCB number
      # aquire information for CpG sites in MCB
      CpGs<-strsplit(MCBset[mcb,'CpGs']," ")[[1]]
      data_used_for_training<-data.frame(allvars = as.ridgemat(t(training_set[CpGs,rz])))
      # train a ridge cox model when variables are more than 5
      if (length(CpGs)<=5){
        ridge_models<-tryCatch(survival::coxph(times ~ allvars,data=data_used_for_training),error = NULL)
      }else{
        ridge_models<-tryCatch(ridge_model(times,data_used_for_training,theta = theta),error = NULL)        
      }
      #predictions
      if (!is.null(ridge_models)) {
        ridge_models$CpGs <- CpGs
        ridge_models <- as.mcb.coxph.penal(ridge_models)
        MCB_cox_matrix_training[mcb,]<-predict.mcb.coxph.penal(ridge_models, data.frame(t(training_set[CpGs,])))
        auc_and_ci = calculate_auc_ci(survival = times,marker = MCB_cox_matrix_training[mcb,rz],predict_time,ci)
        write_MCB['AUC_train']<-auc_and_ci$AUC
        if (ci) write_MCB['95_CI_train']<-auc_and_ci$CI95
        #if it has a independent test set
        if (!is.null(testing_set)){
          MCB_cox_matrix_test_set[mcb,]<-predict.mcb.coxph.penal(ridge_models, data.frame(t(testing_set[CpGs,])))
          auc_and_ci = calculate_auc_ci(Surv.new,marker = MCB_cox_matrix_test_set[mcb,],predict_time,ci)
          write_MCB['AUC_test']<-auc_and_ci$AUC
          if (ci) write_MCB['95_CI_test']<-auc_and_ci$CI95
          if ((write_MCB['AUC_train']+write_MCB['AUC_test'])>best_auc){
            best_auc<-write_MCB['AUC_train']+write_MCB['AUC_test']
            best_model<-list(mcb,ridge_models)
          }
          #if it does not have a independent test set
        }else{
          write_MCB<-write_MCB[1:3]
          if (write_MCB['AUC_train']>best_auc){
            best_auc<-write_MCB['AUC_train']
            best_model<-list(mcb,ridge_models)
          }
        }
      }else{
        stop("This coxph model can not be built.")
      }
      mcb_cox_res<-rbind(mcb_cox_res,write_MCB)
    }
    colnames(mcb_cox_res)<-c("MCB_no","training_set_auc","test_set_auc")
    names(best_model)<-c("MCB_no","cox_model")
    FunctionResults$MCB_cox_matrix_training<-MCB_cox_matrix_training
    FunctionResults$MCB_cox_matrix_test_set<-MCB_cox_matrix_test_set
    FunctionResults$cox_auc_results<-mcb_cox_res
    FunctionResults$maximum_auc<-best_auc
    FunctionResults$best_cox_model<-best_model
  }else if (Method=="enet") {
    # constuction of MCB Method matrix for enet
    MCB_enet_matrix_training<-matrix(0,nrow = nrow(MCBset),ncol = ncol(training_set))
    colnames(MCB_enet_matrix_training)<-colnames(training_set)
    rownames(MCB_enet_matrix_training)<-as.numeric(MCBset[,'MCB_no'])
    #if it has a independent test set create the test_set res set
    if (!is.null(testing_set)) {
      MCB_enet_matrix_test_set<-matrix(0,nrow = nrow(MCBset),ncol = ncol(testing_set))
      colnames(MCB_enet_matrix_test_set)<-colnames(testing_set)
      rownames(MCB_enet_matrix_test_set)<-as.numeric(MCBset[,'MCB_no'])
    }else{
      MCB_enet_matrix_test_set<-NULL
    }
    FunctionResults<-NULL
    rz=!(is.na(Surv)|Surv[,1]==0)
    times=Surv[rz]
    best_auc<-0
    best_model<-NULL
    mcb_enet_res<-NULL
    for (mcb in seq_len(nrow(MCBset))) {
      if (show_bar&!silent){utils::setTxtProgressBar(bar, mcb)}
      write_MCB<-create_doc(ci)
      #save the mcb number
      write_MCB['MCB_no']<-as.numeric(MCBset[mcb,'MCB_no'])
      # build temp variable for saving the results.
      # MCB number
      # aquire information for CpG sites in MCB
      CpGs<-strsplit(MCBset[mcb,'CpGs']," ")[[1]]
      data_used_for_training<-t(training_set[CpGs,rz])
      # train a enet model
      enet_model <- tryCatch(glmnet::cv.glmnet(data_used_for_training,
                                                         times,
                                                         #cox model in enet was used, note that here cox and enet penalty were used.
                                                         family="cox",
                                                         #alpha = 0.5
                                                         alpha=alpha,
                                                         # The elasticnet mixing parameter, with 0≤α≤ 1. The penalty is defined as
                                                         # (1-alpha)/2||beta||_2^2+alpha||beta||_1
                                                         # alpha=1 is the lasso penalty, and alpha=0 the ridge penalty.
                                                         # type.measure = "AUC"
                                                         type.measure= "deviance"
                                                         # It uses AUC as the criterion for 10-fold cross-validation.
                                                         #foldid = 10
      ),error = NULL)
      #predictions
      if (!is.null(enet_model)) {
        correctional_value=1
        while ( sum(stats::coef(enet_model, s = enet_model$lambda.min-0.001*(correctional_value-1))>0)<1 &
               (enet_model$lambda.min-0.001*(correctional_value-1))>0 ) {
          correctional_value=correctional_value*1.25
        }
        lambda_min_corrected<-enet_model$lambda.min-0.001*(correctional_value-1)
        #if you use lambda.1se instead, the penalty of enet would be larger, leading that most of covariates were removed form the final model.
        MCB_enet_matrix_training[mcb,]<-stats::predict(enet_model,t(training_set[CpGs,]),s=lambda_min_corrected)
        auc_and_ci = calculate_auc_ci(survival = times,marker = MCB_enet_matrix_training[mcb,rz],predict_time,ci)
        write_MCB['AUC_train']<-auc_and_ci$AUC
        if (ci) write_MCB['95_CI_train']<-auc_and_ci$CI95
         #if it has a independent test set
        if (!is.null(testing_set)){
          # lambda.min was used.
          MCB_enet_matrix_test_set[mcb,]<-stats::predict(enet_model, t(testing_set[CpGs,]),s=lambda_min_corrected)
          auc_and_ci = calculate_auc_ci(Surv.new,marker = MCB_enet_matrix_test_set[mcb,],predict_time,ci)
          write_MCB['AUC_test']<-auc_and_ci$AUC
          if (ci) write_MCB['95_CI_test']<-auc_and_ci$CI95
          if ((write_MCB['AUC_train']+write_MCB['AUC_test'])>best_auc){
            best_auc<-write_MCB['AUC_train']+write_MCB['AUC_test']
            best_model<-list(mcb,enet_model,lambda_min_corrected)
          }
          #if it does not have a independent test set
        }else{
          write_MCB<-write_MCB[1:3]
          if (write_MCB['AUC_train']>best_auc){
            best_auc<-write_MCB['AUC_train']
            best_model<-list(mcb,enet_model,lambda_min_corrected)
          }
        }
      }else{
        stop("This enet model can not be built.")
      }
      mcb_enet_res<-rbind(mcb_enet_res,write_MCB)
    }
    colnames(mcb_enet_res)<-c("MCB_no","training_set_auc","test_set_auc")
    names(best_model)<-c("MCB_no","enet_model","corrected_lambda(min)")
    FunctionResults$MCB_enet_matrix_training<-MCB_enet_matrix_training
    FunctionResults$MCB_enet_matrix_test_set<-MCB_enet_matrix_test_set
    FunctionResults$enet_auc_results<-mcb_enet_res
    FunctionResults$maximum_auc<-best_auc
    FunctionResults$best_enet_model<-best_model
  }else if (Method=="mboost") {
    # constuction of MCB Method matrix for mboost
    MCB_mboost_matrix_training<-matrix(0,nrow = nrow(MCBset),ncol = ncol(training_set))
    colnames(MCB_mboost_matrix_training)<-colnames(training_set)
    rownames(MCB_mboost_matrix_training)<-as.numeric(MCBset[,'MCB_no'])
    #if it has a independent test set create the test_set res set
    if (!is.null(testing_set)) {
      MCB_mboost_matrix_test_set<-matrix(0,nrow = nrow(MCBset),ncol = ncol(testing_set))
      colnames(MCB_mboost_matrix_test_set)<-colnames(testing_set)
      rownames(MCB_mboost_matrix_test_set)<-as.numeric(MCBset[,'MCB_no'])
    }else{
      MCB_mboost_matrix_test_set<-NULL
    }
    FunctionResults<-NULL
    rz=!(is.na(Surv)|Surv[,1]==0)
    times=Surv[rz]
    best_auc<-0
    best_model<-NULL
    mcb_mboost_res<-NULL
    for (mcb in seq_len(nrow(MCBset))) {
      #if (nrow(MCBset)>1){}
      if (show_bar&!silent) {
        utils::setTxtProgressBar(bar, mcb)
      }
      write_MCB<-create_doc(ci)
      #save the mcb number
      write_MCB['MCB_no']<-as.numeric(MCBset[mcb,'MCB_no'])
      # build temp variable for saving the results.
      # MCB number
      # aquire information for CpG sites in MCB
      CpGs<-strsplit(MCBset[mcb,'CpGs']," ")[[1]]
      data_used_for_training<-t(training_set[CpGs,rz])
      # train a mboost model
      mboost_model <- tryCatch(mboost::glmboost(y=times,x=data_used_for_training,family=mboost::CoxPH(),
                                          control=mboost::boost_control(mstop=n_mstop,nu=n_nu)),error = NULL)
      #predictions
      if (!is.null(mboost_model)) {
        MCB_mboost_matrix_training[mcb,]<-stats::predict(mboost_model, t(training_set[CpGs,]))[,1]
        lp_mboost <-MCB_mboost_matrix_training[mcb,rz]
        hr_model<-survival::coxph(times~lp_mboost)
        auc_and_ci = calculate_auc_ci(survival = times,marker = MCB_mboost_matrix_training[mcb,rz],predict_time,ci)
        write_MCB['AUC_train']<-auc_and_ci$AUC
        if (ci) write_MCB['95_CI_train']<-auc_and_ci$CI95
        #if it has a independent test set
        if (!is.null(testing_set)){
          MCB_mboost_matrix_test_set[mcb,]<-stats::predict(mboost_model, t(testing_set[CpGs,]))[,1]
          auc_and_ci = calculate_auc_ci(Surv.new,marker = MCB_mboost_matrix_test_set[mcb,],predict_time,ci)
          write_MCB['AUC_test']<-auc_and_ci$AUC
          if (ci) write_MCB['95_CI_test']<-auc_and_ci$CI95
          if ((write_MCB['AUC_train']+write_MCB['AUC_test'])>best_auc){
            best_auc<-write_MCB['AUC_train']+write_MCB['AUC_test']
            best_model<-list(mcb,mboost_model,hr_model)
          }
          #if it does not have a independent test set
        }else{
          write_MCB<-write_MCB[1:3]
          if (write_MCB['AUC_train']>best_auc){
            best_auc<-write_MCB['AUC_train']
            best_model<-list(mcb,mboost_model,hr_model)
          }
        }
      }else{
        stop("This mboost model can not be built.")
      }
      mcb_mboost_res<-rbind(mcb_mboost_res,write_MCB)
    }
    colnames(mcb_mboost_res)<-c("MCB_no","training_set_auc","test_set_auc")
    names(best_model)<-c("MCB_no","mboost_model","hr_model")
    FunctionResults$MCB_mboost_matrix_training<-MCB_mboost_matrix_training
    FunctionResults$MCB_mboost_matrix_test_set<-MCB_mboost_matrix_test_set
    FunctionResults$mboost_auc_results<-mcb_mboost_res
    FunctionResults$maximum_auc<-best_auc
    FunctionResults$best_mboost_model<-best_model
  }
  return(FunctionResults)
}

---
title: "EnMCB"
output: 
  rmarkdown::html_vignette:
    toc: true
author: "Xin Yu"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Introduction

This package is designed to help you to create the methylation correlated blocks using methylation profiles. A stacked ensemble of machine learning models, which combined the Cox regression, support vector regression and elastic-net regression model, can be constructed using this package^[Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)]. You also can choose one of them to build DNA methylation signatures associated with disease progression.

Note: This package is still under developing. Some of the functions may change.

Followings are brief insturctions for using this package:

You can install and test our package by downloading source package.

## Installation

```{r echo=TRUE,warning=FALSE,results='hide'}
#if(!requireNamespace("BiocManager", quietly = TRUE))
#    install.packages("BiocManager")
#BiocManager::install("EnMCB")
```

## Useage

First, you need a methylation data set, currently only most common platform 'Illumina Infinium Human Methylation 450K' is supported.

You can use your own datasets,or use our demo data.

You can automatically run following:

```{r echo=TRUE,warning=FALSE,results='hide'}

suppressPackageStartupMessages(library(EnMCB))

methylation_dataset<-create_demo()

res<-IdentifyMCB(methylation_dataset)

```

IdentfyMCB() function will calculated Pearson correlation coefficients between the any two CpGs. A value of Pearson correlation coefficients which under the threshold was used to identify boundaries between any two adjacent markers indicating uncorrelated methylation. Markers not separated by a boundary were combined into the MCB. You can extract the MCB information with,


```{r echo=TRUE,warning=FALSE,results='hide'}

MCB<-res$MCBinformation

```


and select some of MCBs for further modeling.


```{r echo=TRUE,warning=FALSE,results='hide'}

MCB<-MCB[MCB[,"CpGs_num"]>=5,]

```

In order to get differentially methylated blocks, one may run following:

```{r echo=TRUE,warning=FALSE,results='hide'}
#simulation for the group data
groups = c(rep("control",200),rep("dis",255))

DiffMCB_resutls<-DiffMCB(methylation_dataset,
                         groups,
                         MCB)$tab

```


In order to build survival models, one may run following:

```{r echo=TRUE,warning=FALSE,results='hide'}
# sample the dataset into training set and testing set
trainingset<-colnames(methylation_dataset) %in% sample(colnames(methylation_dataset),0.6*length(colnames(methylation_dataset)))

testingset<-!trainingset

#build the models
library(survival)
data(demo_survival_data)

models<-metricMCB(MCB,
                    training_set = methylation_dataset[,trainingset],
                    Surv = demo_survival_data[trainingset],
                    Method = "cox",ci = TRUE)

#select the best
onemodel<-models$best_cox_model$cox_model

```                    

Then, you can predict the risk by the model you build:

```{r echo=TRUE,warning=FALSE,results='hide'}
newcgdata<-data.frame(t(methylation_dataset[,testingset]))
           
prediction_results<-predict(onemodel, newcgdata)
```

In order to build ensemble model, one may run following:

```{r echo=TRUE,warning=FALSE,results='hide'}
# You can choose one of MCBs:
select_single_one=1

em<-ensemble_model(t(MCB[select_single_one,]),
                    training_set=methylation_dataset[,trainingset],
                    Surv_training=demo_survival_data[trainingset])
                    
```

Note that this function only can be used for single MCB only, otherwise the precessing time could be very long.

Then, you can predict the risk by the model you build:

```{r echo=TRUE,warning=FALSE,results='hide'}
em_prediction_results<-ensemble_prediction(ensemble_model = em,
                    prediction_data = methylation_dataset[,testingset])
```

This function will return the single vector with risk scores predicted by ensemble model.

For detailed information, you can find at our references.

## Session Info

```{r echo=TRUE}
sessionInfo()
```

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metricMCB.R
\name{metricMCB}
\alias{metricMCB}
\title{Calculation of the metric matrix for Methylation Correlation Block}
\usage{
metricMCB(MCBset,training_set,Surv,testing_set,
Surv.new,Method,predict_time,ci,silent,alpha,n_mstop,n_nu,theta)
}
\arguments{
\item{MCBset}{Methylation Correlation Block information returned by the IndentifyMCB function.}

\item{training_set}{methylation matrix used for training the model in the analysis.}

\item{Surv}{Survival function contain the survival information for training.}

\item{testing_set}{methylation matrix used in the analysis. This can be missing then training set itself will be used as testing set.}

\item{Surv.new}{Survival function contain the survival information for testing.}

\item{Method}{model used to calculate the compound values for multiple Methylation correlation blocks. \cr
Options include "svm" "cox" "mboost" and "enet". The default option is SVM method.}

\item{predict_time}{time point of the ROC curve used in the AUC calculations, default is 5 years.}

\item{ci}{if True, the confidence intervals for AUC under area under the receiver operating characteristic curve will be calculated. This will be time consuming. default is False.}

\item{silent}{True indicates that processing information and progress bar will be shown.}

\item{alpha}{The elasticnet mixing parameter, with 0 <= alpha <= 1. alpha=1 is the lasso penalty, and alpha=0 the ridge penalty. \cr
It works only when "enet" Method is selected.}

\item{n_mstop}{an integer giving the number of initial boosting iterations. If mstop = 0, the offset model is returned. \cr
It works only when "mboost" Method is selected.}

\item{n_nu}{a double (between 0 and 1) defining the step size or shrinkage parameter in mboost model. \cr
It works only when "mboost" Method is selected.}

\item{theta}{penalty used in the penalized coxph model, which is theta/2 time sum of squared coefficients. default is 1. \cr
It works only when "cox" Method is selected.}
}
\value{
Object of class \code{list} with elements (XXX will be replaced with the model name you choose):
 \tabular{ll}{
   \code{MCB_XXX_matrix_training} \tab Prediction results of model for training set. \cr
   \code{MCB_XXX_matrix_test_set} \tab Prediction results of model for test set. \cr
   \code{XXX_auc_results} \tab AUC results for each model. \cr
   \code{best_XXX_model} \tab Model object for the model with best AUC. \cr
   \code{maximum_auc} \tab Maximum AUC for the whole generated models. \cr
 }
}
\description{
To enable quantitative analysis of the methylation patterns \cr
within individual Methylation Correlation Blocks across many samples, a single metric to \cr
define the methylated pattern of multiple CpG sites within each block.\cr
Compound scores which calculated all CpGs within individual Methylation Correlation Blocks by linear, SVM or elastic-net model\cr
Predict values were used as the compound methylation values of Methylation Correlation Blocks.
}
\examples{
#import datasets
data(demo_survival_data)
datamatrix<-create_demo()
data(demo_MCBinformation)
#select MCB with at least 3 CpGs.
demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]

trainingset<-colnames(datamatrix) \%in\% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
testingset<-!trainingset
#create the results using Cox regression. 
mcb_cox_res<-metricMCB(MCBset = demo_MCBinformation,
               training_set = datamatrix[,trainingset],
               Surv = demo_survival_data[trainingset],
               testing_set = datamatrix[,testingset],
               Surv.new = demo_survival_data[testingset],
               Method = "cox"
               )

}
\references{
Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
}
\author{
Xin Yu
}
\keyword{Correlation}
\keyword{Methylation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{predict.mcb.coxph.penal}
\alias{predict.mcb.coxph.penal}
\title{predict coxph penal using MCB}
\usage{
\method{predict}{mcb.coxph.penal}(object, newdata, ...)
}
\arguments{
\item{object}{the results of a coxph fit.}

\item{newdata}{Optional new data at which to do predictions. If absent predictions are for the data frame used in the original fit. 
When coxph has been called with a formula argument created in another context, i.e., coxph has been called within another function and the formula was passed as an argument to that function, 
there can be problems finding the data set. See the note below.}

\item{...}{other parameters pass to predict.coxph}
}
\value{
prediction values of regression.
}
\description{
Compute fitted values and regression terms for a model fitted by coxph
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{demo_data}
\alias{demo_data}
\title{Expression matrix of demo dataset.}
\format{
ExpressionSet:
\describe{
  \item{rownames}{rownames of 10020 CpG features}
  \item{colnames}{colnames of 455 samples}
  \item{realdata}{Real data matrix for demo.}
}
}
\usage{
data(demo_data)
}
\description{
A Expression matrix containing the 10020 CpGs beta value of 455 samples in TCGA lung Adenocarcinoma dataset.
This will call from create_demo() function.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pre_process_methylation.R
\name{pre_process_methylation}
\alias{pre_process_methylation}
\title{Preprocess the Beta value matrix}
\usage{
pre_process_methylation(met,Mvalue,constant_offset,remove_na,remove_percentage)
}
\arguments{
\item{met}{methylation matrix for CpGs. Rows are the CpG names, columns are samples.}

\item{Mvalue}{Boolean value, TRUE for the M transformation.}

\item{constant_offset}{the constant offset used in the M transformation formula.}

\item{remove_na}{Boolean value, if TRUE ,CpGs with NA values will be removed.}

\item{remove_percentage}{If precentage of NA value exceed the threshold(percentage), the whole CpG probe will be removed. Otherwise, the NA values are replaced with rowmeans.}
}
\value{
Object of class \code{matrix}.
}
\description{
This process is optional for the pipeline.
This function pre-process the Beta matrix and transform the Beta value into M value.
}
\examples{
demo_set<-create_demo()
pre_process_methylation(demo_set,Mvalue=FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CompareMCB.R
\name{CompareMCB}
\alias{CompareMCB}
\title{Compare multiple methylation correlated blocks lists}
\usage{
CompareMCB(
  MCBs,
  method = c("attractors")[1],
  p_value = 0.05,
  min_CpGs = 5,
  platform = "Illumina Methylation 450K"
)
}
\arguments{
\item{MCBs}{Methylation correlated blocks list.}

\item{method}{method used for calculation of differential expression, \cr
should be one of "attractors","t-test". Defualt is "attractors".}

\item{p_value}{p value threshold for the test.}

\item{min_CpGs}{threshold for minimum CpGs must included in the individual MCBs.}

\item{platform}{This parameter indicates the platform used to produce the methlyation profile.}
}
\value{
Object of class \code{list} with elements:
 \tabular{ll}{
   \code{MCBsites} \tab Character set contains all CpG sites in MCBs. \cr
   \code{MCBinformation} \tab Matrix contains the information of results. \cr
 }
}
\description{
This function is used to find the Methylation correlated blocks that differentially expressed between groups. 
This function calculates attractors of all the MCBs among the groups and find the attractor MCBs. \cr
}
\details{
Currently, only illumina 450k platform is supported, the methylation profile need to convert into matrix format.
}
\examples{
data('demo_data',package = "EnMCB")


}
\references{
Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{demo_survival_data}
\alias{demo_survival_data}
\title{Survival data of demo dataset.}
\format{
Surv data created by Surv() function in survival package.
This data have two unnamed arguments, they will match time and event.
}
\usage{
data(demo_survival_data)
}
\description{
A Surv containing survival value of 455 samples in TCGA lung Adenocarcinoma dataset.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{demo_MCBinformation}
\alias{demo_MCBinformation}
\title{MCB information.}
\format{
A data frame with 94 rows and 8 variables:
\describe{
  \item{MCB_no}{MCB code}
  \item{start}{Start point of this MCB in the chromosome.}
  \item{end}{End point of this MCB in the chromosome.}
  \item{CpGs}{All the CpGs probe names in the MCB.}
  \item{location}{Start, end point and the chromosome number of this MCB.}
  \item{chromosomes}{the chromosome number of this MCB.}
  \item{length}{the length of bps of this MCB in the chromosome.}
  \item{CpGs_num}{number of CpG probes of this MCB.}
}
}
\usage{
data(demo_MCBinformation)
}
\description{
A dataset containing the number and other attributes of 94 MCBs; 
This results was created by the identification function IdentifyMCB.
This data used for metricMCB function.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensemble_model.R
\name{ensemble_model}
\alias{ensemble_model}
\title{Trainging stacking ensemble model for Methylation Correlation Block}
\usage{
ensemble_model(single_res,training_set,Surv_training,testing_set,
Surv_testing,ensemble_type)
}
\arguments{
\item{single_res}{Methylation Correlation Block information returned by the IndentifyMCB function.}

\item{training_set}{methylation matrix used for training the model in the analysis.}

\item{Surv_training}{Survival function contain the survival information for training.}

\item{testing_set}{methylation matrix used for testing the model in the analysis.}

\item{Surv_testing}{Survival function contain the survival information for testing.}

\item{ensemble_type}{Secondary model use for ensemble, one of "Cox", "C-index" and "feature weighted linear regression".
"feature weighted linear regression" only uses two meta-features namely kurtosis and S.D.}
}
\value{
Object of class \code{list} with elements (XXX repesents the model you choose):
 \tabular{ll}{
   \code{cox} \tab Model object for the cox model at first level. \cr
   \code{svm} \tab Model object for the svm model at first level. \cr
   \code{enet} \tab Model object for the enet model at first level. \cr
   \code{mboost} \tab Model object for the mboost model at first level. \cr
   \code{stacking} \tab Model object for the stacking model. \cr
 }
}
\description{
Method for training a stacking ensemble model for Methylation Correlation Block.
}
\examples{
#import datasets
library(survival)
data(demo_survival_data)
datamatrix<-create_demo()
data(demo_MCBinformation)
#select MCB with at least 3 CpGs.
demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]
trainingset<-colnames(datamatrix) \%in\% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
select_single_one=1
em<-ensemble_model(t(demo_MCBinformation[select_single_one,]),
    training_set=datamatrix[,trainingset],
    Surv_training=demo_survival_data[trainingset])


}
\references{
Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
}
\author{
Xin Yu
}
\keyword{ensemble}
\keyword{methylation}
\keyword{stacking}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/univ_coxph.R
\name{univ_coxph}
\alias{univ_coxph}
\title{univariate and multivariate survival analysis using coxph}
\usage{
univ_coxph(dataframe, y_surv, digits = 4, asnumeric = TRUE)
}
\arguments{
\item{dataframe}{Clinic data and covariates ready to be tested. Rows are variables and columns are samples.}

\item{y_surv}{Survival function contain survival data, usually are obtained form Surv() function in survival package.}

\item{digits}{Integer indicating the number of decimal places.}

\item{asnumeric}{indicator that the data will be (True) / not (False) transformed into numeric. Default is true.}
}
\value{
Object of class \code{matrix} with results.
}
\description{
univariate and multivariate survival analysis using coxph
}
\examples{
data(demo_survival_data)
data('demo_data',package = "EnMCB")
demo_set<-demo_data$realdata
res<-univ_coxph(demo_set,demo_survival_data)
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as.data.frame.ridgemat}
\alias{as.data.frame.ridgemat}
\title{data frame ridge matrix}
\usage{
\method{as.data.frame}{ridgemat}(x, ...)
}
\arguments{
\item{x}{data vector}

\item{...}{other parameters pass to as.data.frame.model.matrix()}
}
\description{
data frame ridge matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{create_demo}
\alias{create_demo}
\title{create demo matrix}
\usage{
create_demo(model = c("all", "short")[1])
}
\arguments{
\item{model}{Two options, 'all' or 'short' for creating full dataset or very brief demo.}
}
\value{
This function will generate a demo data.
}
\description{
Demo matrix for methylation matrix.
}
\examples{
demo_set<-create_demo()
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fast_roc_calculation.R
\name{fast_roc_calculation}
\alias{fast_roc_calculation}
\title{Fast calculation of AUC for ROC using parallel strategy}
\usage{
fast_roc_calculation(test_matrix, y_surv, predict_time = 5, roc_method = "NNE")
}
\arguments{
\item{test_matrix}{Test matrix used in the analysis. Colmuns are samples, rows are markers.}

\item{y_surv}{Survival information created by Surv function in survival package.}

\item{predict_time}{Time point of the ROC curve, default is 5 year.}

\item{roc_method}{Method for fitting joint distribution of (marker,t), either of KM or NNE, the default method is NNE.}
}
\value{
This will retrun a numeric vector contains AUC results for each row in test_matrix.
}
\description{
This function is used to create time-dependent ROC curve from censored survival data using the Kaplan-Meier (KM) or Nearest Neighbor Estimation (NNE) method of Heagerty, Lumley and Pepe, 2000
}
\examples{
data(demo_survival_data)
data('demo_data',package = "EnMCB")
demo_set<-demo_data$realdata
res<-fast_roc_calculation(demo_set[1:2,],demo_survival_data)

}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IdentifyMCB_parallel.R
\name{IdentifyMCB_parallel}
\alias{IdentifyMCB_parallel}
\title{Identification of methylation correlated blocks with parallel algorithm}
\usage{
IdentifyMCB_parallel(
  MethylationProfile,
  method = c("pearson", "spearman", "kendall")[1],
  CorrelationThreshold = 0.8,
  PositionGap = 1000,
  platform = "Illumina Methylation 450K",
  verbose = T
)
}
\arguments{
\item{MethylationProfile}{Methylation matrix is used in the analysis.}

\item{method}{method used for calculation of correlation, \cr
should be one of "pearson","spearman","kendall". Defualt is "pearson".}

\item{CorrelationThreshold}{coef correlation threshold is used for define boundaries.}

\item{PositionGap}{CpG Gap between any two CpGs positioned CpG sites less than 1000 bp (default) will be calculated.}

\item{platform}{This parameter indicates the platform used to produce the methlyation profile. 
You can use your own annotation file.}

\item{verbose}{True as default, which will print the block information for each chromosome.}
}
\value{
Object of class \code{list} with elements:
 \tabular{ll}{
   \code{MCBsites} \tab Character set contains all CpG sites in MCBs. \cr
   \code{MCBinformation} \tab Matrix contains the information of results. \cr
 }
}
\description{
This function is used to partition the genome into blocks of tightly co-methylated CpG sites, \cr
Methylation correlated blocks parallelly. This function calculates Pearson correlation coefficients between \cr
the beta values of any two CpGs < CorrelationThreshold was used to identify boundaries between any two \cr
adjacent markers indicating uncorrelated methylation. Markers not separated by a boundary were combined into MCB. \cr
Pearson correlation coefficients between two adjacent CpGs were calculated.
}
\details{
Currently, only illumina 450k platform is supported, the methylation profile need to convert into matrix format.
}
\examples{
data('demo_data',package = "EnMCB")

#import the demo TCGA data with 10000+ CpGs site and 455 samples
#remove # to run
res<-IdentifyMCB_parallel(demo_data$realdata)
demo_MCBinformation<-res$MCBinformation


}
\references{
Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metricMCB_cv.R
\name{metricMCB.cv}
\alias{metricMCB.cv}
\title{Calculation of model AUC for Methylation Correlation Blocks using cross validation}
\usage{
metricMCB.cv(MCBset,data_set,Surv,nfold,
Method,predict_time,alpha,n_mstop,n_nu,theta,silent)
}
\arguments{
\item{MCBset}{Methylation Correlation Block information returned by the IndentifyMCB function.}

\item{data_set}{methylation matrix used for training the model in the analysis.}

\item{Surv}{Survival function contain the survival information for training.}

\item{nfold}{fold used in the cross validation precedure.}

\item{Method}{model used to calculate the compound values for multiple Methylation correlation blocks. Options include "svm", "cox", "mboost", and "enet". The default option is SVM method.}

\item{predict_time}{time point of the ROC curve used in the AUC calculations, default is 3 years.}

\item{alpha}{The elasticnet mixing parameter, with 0 <= alpha <= 1. alpha=1 is the lasso penalty, and alpha=0 the ridge penalty. It works only when "enet" Method is selected.}

\item{n_mstop}{an integer giving the number of initial boosting iterations. If mstop = 0, the offset model is returned. It works only when "mboost" Method is selected.}

\item{n_nu}{a double (between 0 and 1) defining the step size or shrinkage parameter in mboost model. It works only when "mboost" Method is selected.}

\item{theta}{penalty used in the penalized coxph model, which is theta/2 time sum of squared coefficients. default is 1. It works only when "cox" Method is selected.}

\item{silent}{Ture indicates that processing information and progress bar will be shown.}
}
\value{
Object of class \code{list} with elements (XXX will be replaced with the model name you choose):
 \tabular{ll}{
   \code{MCB_matrix} \tab Prediction results of model. \cr
   \code{auc_results} \tab AUC results for each model. \cr
 }
}
\description{
To enable quantitative analysis of the methylation patterns
within individual Methylation Correlation Blocks across many samples, a single metric to
define the methylated pattern of multiple CpG sites within each block.
Compound scores which calculated all CpGs within individual Methylation Correlation Blocks by SVM model
were used as the compound methylation values of Methylation Correlation Blocks.
}
\examples{
#import datasets
data(demo_survival_data)
datamatrix<-create_demo()
data(demo_MCBinformation)
#select MCB with at least 3 CpGs.
demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]

trainingset<-colnames(datamatrix) \%in\% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
testingset<-!trainingset
#create the results using Cox regression. 
mcb_cox_res<-metricMCB.cv(MCBset = demo_MCBinformation,
               data_set = datamatrix,
               Surv = demo_survival_data,
               Method = "cox")

}
\references{
Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
}
\author{
Xin Yu
}
\keyword{Correlation}
\keyword{Methylation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as.ridgemat}
\alias{as.ridgemat}
\title{ridge matrix}
\usage{
as.ridgemat(x)
}
\arguments{
\item{x}{data vector}
}
\description{
as.matrix attempts to turn its argument
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensemble_prediction.R
\name{ensemble_prediction}
\alias{ensemble_prediction}
\title{fitting function using stacking ensemble model for Methylation Correlation Block}
\usage{
ensemble_prediction(ensemble_model, prediction_data, multiple_results = FALSE)
}
\arguments{
\item{ensemble_model}{ensemble model which built by ensemble_model() function}

\item{prediction_data}{A vector, matrix, list, or data frame containing the predictions (input).}

\item{multiple_results}{Boolean vector, True for including the single model results.}
}
\value{
Object of numeric class \code{double}
}
\description{
predict is a generic function for predictions from the results of stacking ensemble model fitting functions.
The function invokes particular methods which is the ensemble model described in the reference.
}
\examples{
library(survival)
#import datasets
data(demo_survival_data)
datamatrix<-create_demo()
data(demo_MCBinformation)
#select MCB with at least 3 CpGs.
demo_MCBinformation<-demo_MCBinformation[demo_MCBinformation[,"CpGs_num"]>2,]
trainingset<-colnames(datamatrix) \%in\% sample(colnames(datamatrix),0.6*length(colnames(datamatrix)))
testingset<-!trainingset
#select one MCB
select_single_one=1
em<-ensemble_model(t(demo_MCBinformation[select_single_one,]),
    training_set=datamatrix[,trainingset],
    Surv_training=demo_survival_data[trainingset])

em_prediction_results<-ensemble_prediction(ensemble_model = em,
prediction_data = datamatrix[,testingset])

}
\references{
Xin Yu et al. 2019 Predicting disease progression in lung adenocarcinoma patients based on methylation correlated blocks using ensemble machine learning classifiers (under review)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiffMCB.R
\name{DiffMCB}
\alias{DiffMCB}
\title{Differential expressed methylation correlated blocks}
\usage{
DiffMCB(
  methylation_matrix,
  class_vector,
  mcb_matrix = NULL,
  min.cpgsize = 5,
  pVals_num = 0.05,
  base_method = c("Fstat", "Tstat", "eBayes")[1],
  sec_method = c("ttest", "kstest")[1],
  ...
)
}
\arguments{
\item{methylation_matrix}{methylation profile matrix.}

\item{class_vector}{class vectors that indicated the groups.}

\item{mcb_matrix}{dataframe or matrix results returned by IdentifyMCB function.}

\item{min.cpgsize}{threshold for minimum CpGs must included in the individual MCBs.}

\item{pVals_num}{p value threshold for the test.}

\item{base_method}{base method used for calculation of differentially methylated regions,
should be one of 'Fstat','Tstat','eBayes'. Defualt is Fstat.}

\item{sec_method}{secondly method in attractor framework, should be one of 'kstest','ttest'. Defualt is ttest.}

\item{...}{other parameters pass to the function.}
}
\value{
Object of class \code{list} with elements:
 \tabular{ll}{
   \code{global} \tab Character set contains statistical value for all CpG sites in MCBs. \cr
   \code{tab} \tab Matrix contains the information of results. \cr
 }
}
\description{
This function is used to find the Methylation correlated blocks that differentially expressed between groups based on the attractor framework. 
This function calculates attractors of all the MCBs among the groups and find the attractor MCBs. \cr
}
\details{
Currently, only illumina 450k platform is supported. \cr
If you want to use other platform, please provide the annotation file with CpG's chromosome and loci. \cr
The methylation profile need to convert into matrix format.
}
\examples{
data('demo_data', package = "EnMCB")
data('demo_survival_data', package = "EnMCB")
data('demo_MCBinformation', package = "EnMCB")
#Using survival censoring as group label just for demo, 
#this may replace with disease and control group in real use.
diffMCB_results <- DiffMCB(demo_data$realdata,demo_survival_data[,2], 
                           demo_MCBinformation,
                           pVals_num = 1)

}
\references{
Xin Yu, De-Xin Kong, EnMCB: an R/bioconductor package for predicting disease progression based on methylation correlated blocks using ensemble models, Bioinformatics, 2021, btab415
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi_coxph.R
\name{multi_coxph}
\alias{multi_coxph}
\title{multivariate survival analysis using coxph}
\usage{
multi_coxph(dataframe, y_surv, digits = 4, asnumeric = TRUE)
}
\arguments{
\item{dataframe}{Clinic data and covariates ready to be tested. Note that Rows are samples and columns are variables.}

\item{y_surv}{Survival function contain survival data, usually are obtained form Surv() function in survival package.}

\item{digits}{Integer indicating the number of decimal places.}

\item{asnumeric}{indicator that the data will be (True) / not (False) transformed into numeric. Default is true.}
}
\value{
Object of class \code{matrix} with results.
}
\description{
multivariate survival analysis using coxph
}
\examples{
data(demo_survival_data)
data('demo_data',package = "EnMCB")
demo_set<-demo_data$realdata
res<-multi_coxph(t(demo_set),demo_survival_data)
}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{draw_survival_curve}
\alias{draw_survival_curve}
\title{draw survival curve}
\usage{
draw_survival_curve(
  exp,
  living_days,
  living_events,
  write_name,
  title_name = "",
  threshold = NA,
  file = FALSE
)
}
\arguments{
\item{exp}{expression level for variable.}

\item{living_days}{The survival time (days) for each individual.}

\item{living_events}{The survival event for each individual, 0 indicates alive and 1 indicates death.
Other choices are TRUE/FALSE (TRUE = death) or 1/2 (2=death). For interval censored data, the status indicator is 0=right censored, 1=event at time, 2=left censored, 3=interval censored.}

\item{write_name}{The name for pdf file which contains the result figure.}

\item{title_name}{The title for the result figure.}

\item{threshold}{Threshold used to indicate the high risk or low risk.}

\item{file}{If True, function will automatic generate a result pdf, otherwise it will
return a ggplot object. Default is FALSE.}
}
\value{
This function will generate a pdf file with 300dpi which compare survival curves using the Kaplan-Meier (KM) test.
}
\description{
Draw a survival curve based on survminer package. This is a wrapper function of ggsurvplot.
}
\examples{
data(demo_survival_data)
library(survival)
demo_set<-create_demo()
draw_survival_curve(demo_set[1,],
    living_days = demo_survival_data[,1],
    living_events =demo_survival_data[,2],
    write_name = "demo_data" )

}
\author{
Xin Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{anno_matrix}
\alias{anno_matrix}
\title{IlluminaHumanMethylation450kanno}
\format{
IlluminaHumanMethylation450kanno.ilmn12.hg19 annotation file.
This data have several columns
}
\usage{
data(anno_matrix)
}
\description{
IlluminaHumanMethylation450kanno
}
\keyword{datasets}
